@@ -1,14 +1,14 @@
-
-(define Denemo #t)
-
 (use-modules (srfi srfi-1))
 (use-modules (srfi srfi-8))
 (use-modules (srfi srfi-13))
 (use-modules (ice-9 regex))
 (use-modules (ice-9 optargs))
-(load "ans-7.scm")
+(load "ans.scm")
 (load "notationmagick.scm")
 
+;Needed to see if lyimport / mxml import is called from inside or outside Denemo
+(define Denemo #t)
+
 ; Create a seed for (random int) one time Denemo starts. The seed is altered by random itself afterwards.
 (let ((time (gettimeofday)))
     (set! *random-state*
@@ -17,6 +17,53 @@
 
 (define DenemoKeypressActivatedCommand #f);;;is true while a keyboard shortcut is invoking a script, unless the script has set it to #f
 
+
+;;;;;; Helper functions for scripters
+(define (blank)
+	(system "clear"))
+
+(define disp (lambda args
+   (letrec ((disp-in (lambda (arg) 
+              (if (null? arg) 
+                  #f 
+                  (begin 
+                     (display (car arg)) 
+                     (disp-in (cdr arg))))))) 
+		     (disp-in args)
+		     (newline))))
+
+
+
+;;;;;;;;;;;Get Lilypond Objects as strings. Currently just manual converting for single cases.
+(define (GetContextAsLilypond) ; You will likely need (GetTypeAsLilypond) also. TODO: Create a real function!
+"Staff"
+)
+
+(define (GetTypeAsLilypond)   ; You will likely need (GetContextAsLilypond) also. TODO: Replace with real information, derived from Lilypond
+(define type (string->symbol (d-GetType)))
+	(case type  ; Convert Denemo type to lilypond type
+		((TIMESIG) "TimeSignature")	
+		((CHORD) "NoteHead") ; Rests will be detected as CHORD but it will not work
+		((KEYSIG) "KeySignature")
+		((CLEF) "Clef")
+		(else #f) 
+	)
+)
+
+
+;;;;;;;;;; Prototype to insert Lilypond Directives. Wants a pair with car Tag and cdr lilypond: (cons "BreathMark" "\\breathe")
+(define* (StandAloneDirectiveProto pair #:optional (step? #t))
+	(d-Directive-standalone (car pair))
+	(d-DirectivePut-standalone-postfix (car pair) (cdr pair))
+	(d-DirectivePut-standalone-display(car pair) (car pair))
+	(d-DirectivePut-standalone-minpixels (car pair) 30)
+	(if step?
+		(d-MoveCursorRight)
+	)
+	(d-RefreshDisplay)
+)
+
+
 ;;;;;;;;;; create documentation for a command - this version just prints out basic info
 ;;;;;;;;;;;;;DocumentCommand
 (define (DocumentCommand name)
@@ -77,19 +124,33 @@
 
 (define (NextSelectedObjectAllStaffs)
 	(if  (not (d-NextSelectedObject)) 
-		(if  (and (d-MoveToStaffDown) (d-NextSelectedObject) (d-PrevSelectedObject))
-		(begin 
-		 (let loop ()
-			(if (d-PrevSelectedObject)
-				(loop)))
-		#t); block end. 
-		#f ; if !StaffDown
-		); fi StaffDown
-	#t ;NextSelecetedObject was succesful
+	  (if  (and (d-MoveToStaffDown) (d-IsInSelection))
+		 (selection::MoveToStaffBeginning) ; there is a selection a staff down, loop to the beginning
+		#f ; there is no staff down or the selection is single staff.
+		)
+	  #t ;NextSelecetedObject was succesful
 	)
 )
 
 
+(define (selection::MoveToStaffBeginning)
+(d-PushPosition)
+ (if (d-GoToSelectionStart) ; Test if there is a selection at all
+	(begin
+		(d-PopPosition); return to the initial position to go to the correct staff
+		(d-PushPosition); save it again in case that there is no selection in this staff.
+		(d-MoveToBeginning)
+		(let loop ()  ; Real work begins here. Loop through until you found it or end of staff.
+		  (if (d-IsInSelection) 
+		  	#t ; found the first note
+			(if (d-NextObject) (loop) ; prevent endless loop if reaching the end of a staff without selection present
+				(begin  (d-PopPosition) #f ))))  ; if end of staff and no selection return to initial position and return #f
+	)
+	(begin  (d-PopPosition) #f )   ; no selection at all. 
+	) ; fi GoToSelectionStart
+)
+
+
 ;SingleAndSelectionSwitcher by Nils Gey Jan/2010
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;Automatically applies a script to a whole selection. You can give different commands or command blocks with (begin) for single items or whole selections. You can enter a complete scheme script with (),  arguments and everything you would want to run standalone. Don't forget to escape chars like  \" . You can even use a complete (begin ) block.
@@ -441,10 +502,10 @@
 	  (if (regexp-match? thematch)
 	      (set! current (match:substring thematch 1)))))
     (if (boolean? title)
-	(set! title (d-GetUserInput (string-append type " " fieldname)
-				    (string-append "Give a name for the " fieldname " of the " type) current)))
+	(set! title (scheme-escape (d-GetUserInput (string-append type " " fieldname)
+				    (string-append "Give a name for the " fieldname " of the " type) current))))
     (d-DirectivePut-header-override tag DENEMO_OVERRIDE_GRAPHIC)
-    (d-DirectivePut-header-display tag (string-append type " " fieldname ": " title))
+    (d-DirectivePut-header-display tag (string-append type " " fieldname ": " (html-escape title)))
     
     (d-DirectivePut-header-postfix tag (string-append field " = \"" title "\"\n"))))
 
@@ -462,8 +523,8 @@
 	;;(display thematch)
 	(if (regexp-match? thematch)
 	    (set! current (match:substring thematch 1)))))
-  (set! title (d-GetUserInput (string-append "Score " field) 
-			      (string-append "Give a name for the " field " of the whole score") current))
+  (set! title (scheme-escape (d-GetUserInput (string-append "Score " field) 
+			      (string-append "Give a name for the " field " of the whole score") current)))
   (d-DirectivePut-scoreheader-override tag DENEMO_OVERRIDE_GRAPHIC)
   (d-DirectivePut-scoreheader-display tag (string-append field ": " title))
   (d-DirectivePut-scoreheader-postfix tag (string-append field " = \"" title "\"\n"))))
@@ -1039,16 +1100,21 @@
 
 
 
-;;;; Move right until "appending" or "none" which is the Measure End
+;;;; GoToMeasureEnd: Move right until "appending" or "none" which is the Measure End
 (define (GoToMeasureEnd)
-
-(let loop ()
+  (let loop ()
     (if  (or (string-ci=?  (d-GetType) "none") (string-ci=?  (d-GetType) "appending"))
 	#t
-	(begin (d-MoveCursorRight) (loop))))
-		
+	(begin (d-MoveCursorRight) (loop))))	
 )
 
+;;;; GoToMeasureBeginning
+(define (GoToMeasureBeginning)
+  (if (d-MoveToMeasureLeft)
+	(d-MoveToMeasureRight)  
+	(d-MoveToBeginning) 
+  )
+)
 
 ;;;;;;;;;;;;;;;;;;;; Paste by Nils Gey // 2010
 ; Multistaff-Pasting always adds the complete part AFTER the current measure. It will never paste into an existing measure, not even in empty ones. 
@@ -1278,9 +1344,212 @@
 ;;;;;;;;;;;;;;; End Shuffle
 
 
-
 (define (lyimport::load-file pathname filename)
   (load (string-append DENEMO_ACTIONS_DIR "lyimport.scm"))
   (set! lyimport::pathname pathname) 
   (set! lyimport::filename filename)
   (eval-string (lyimport::import)))
+
+;;;;;;;;;;Duration Conversions between Denemo, Lilypond and Tick syntax.
+;; A table of common values
+; 6 = 256 = 8
+;12 = 128 = 7
+;24 = 64 = 6
+;48 = 32 = 5
+;96 = 16 = 4
+;192 = 8 = 3
+;384 =4 = 2
+;768 = 2 = 1
+;1536 = 1 = 0
+;3072 = 0.5  = -1  ; Breve. 0.5 and -1 are not existend.  Lilypond and Denemo use the string breve instead.
+
+; Guile returns a value with .0, which should be exact but internally it's inexact. So we need this little back and forth conversion hack.
+(define (duration::inexact->exact return)
+  (inexact->exact (string->number (number->string return)))
+)
+
+;Some functions, like Upbeat, know only a single tick-value. They need to guess the baseNote.
+(define (duration::GuessBaseNoteInTicks ticks)
+; first guess the basic note duration.  2*x > ticks < 1*x  is always true in our circumstance 
+  (cond
+	 ( (and (>= ticks 6) (< ticks 12)) 6 )       ;1/256
+ 	 ( (and (>= ticks 12) (< ticks 24))  12 )   ;1/128
+ 	 ( (and (>= ticks 24) (< ticks 48)) 24)   ;1/64
+ 	 ( (and (>= ticks 48) (< ticks 96)) 48 )   ;1/32
+ 	 ( (and (>= ticks 96) (< ticks 192))  96)   ;sixteen 1/16
+ 	 ( (and (>= ticks 192) (< ticks 384)) 192 ) ; eight 1/8
+ 	 ( (and (>= ticks 384) (< ticks 768))  384 ) ; quarter 1/4
+ 	 ( (and (>= ticks 768) (< ticks 1536))  768) ; half 1/2
+ 	 ( (and (>= ticks 1536) (< ticks 3072))  1536 ) ; whole 1
+ 	 ( (and (>= ticks 3072) (< ticks 6144))  3072 ) ; breve 2*1
+ 	 ( (and (>= ticks 6144) (< ticks 12288))  6144) ; longa 4*1
+ 	 ( (and (>= ticks 12288) (< ticks 24576))  12288 )  ; maxima 8*1
+	(else #f)
+))
+
+; Calculate a new Duration in Ticks with a basic note value and a number of augmentation-dots
+(define (duration::CalculateTicksWithDots baseTicks numberOfDots)
+	; x = basic note value
+	; n = number of dots
+	; 2x - x/2^n 
+	(-  (* 2 baseTicks) (/ baseTicks (expt 2 numberOfDots)))
+)
+
+; Calculate how many dots a tick value has. Needs base duration, too.
+(define (duration::CalculateDotsFromTicks ticks base)
+; x = base , y = ticks. result is the number of dots
+; log(x/(2*x-y))  / log(2)
+ (define return (/  (log (/ base (- (* base 2) ticks)))   (log 2) ))
+ (duration::inexact->exact return)
+)
+
+; Get Number of Dots from a Lilypond string like "2.". Its so basic it will work on Denemo notes, too.
+(define (duration::GetNumberOfDotsInLilypond input)
+ (length (cdr (string-split input #\.)))
+)
+
+; For the sake of completenes. Denemo and Lilypond dot-syntax is just the same, only the number itself is different.
+(define (duration::GetNumberOfDotsInDenemo input)
+	(duration::GetNumberOfDotsInLilypond input)
+)
+
+(define (duration::denemo->lilypond number)
+	(define return (expt 2 number))
+	return
+)
+
+(define (duration::lilypond->denemo number)
+	(define return (/ (log number) (log 2))	)
+	 (duration::inexact->exact return)
+)
+
+(define (duration::denemo->ticks number) ; increases with negative integers
+	(define return (* (expt 2 (- 8 number)) 6))
+	return
+)
+
+(define (duration::lilypond->ticks number) ; increases with 0.5, 0.25 etc.
+	(define return (* (expt 2 (- 8 (/ (log number) (log 2)))) 6))
+	 (duration::inexact->exact return)
+)
+
+; Ticks->Denemo wants a number but returns a string because of dots
+(define* (duration::ticks->denemo number #:optional (basenumber number))
+ (define numberOfDots  (duration::CalculateDotsFromTicks number basenumber))
+;n = -(log(y/3)-9*log(2))/log(2) 
+ (define return (- (/ (- (log (/ basenumber 3)) (* 9 (log 2))) (log 2))))
+ (set! return (duration::inexact->exact return))
+ (string-append (number->string return) (string-concatenate (make-list numberOfDots "."))) 
+)
+
+;Ticks->Lilypond wants a number but returns a string because of dots
+(define* (duration::ticks->lilypond number #:optional (basenumber number))
+ ;Equation in readable form: http://www.numberempire.com/equationsolver.php?function=y+%3D+6*2^%288-n%29&var=n&answers=
+ (define numberOfDots  (duration::CalculateDotsFromTicks number basenumber))
+ (define return  (expt 2 (- (/ (- (log (/ basenumber 3)) (* 9 (log 2))) (log 2)))))
+ (set! return (duration::inexact->exact return))
+ (string-append (number->string return) (string-concatenate (make-list numberOfDots "."))) 
+)
+;;;;;;;;;; End of duration-conversion
+
+
+;;;;;;;; Applied duration scripts
+(define (duration::GetNumberOfDotsInTicks) ; Fails for tuplets
+	 (duration::CalculateDotsFromTicks (d-GetDurationInTicks) (duration::GetBaseDurationInTicks))
+)
+	   
+
+(define (duration::GetBaseDurationInTicks)
+	(define ticks (d-GetBaseDurationInTicks))
+	(if ticks
+		(abs ticks)
+		#f)
+)
+	   
+
+(define (duration::GetSelectionDurationInTicks) 
+  (d-PushPosition)
+  (if (d-GoToSelectionStart)
+	(let loop ((ticks 0))
+		(set! ticks (+ ticks (d-GetDurationInTicks)))
+		(if (d-NextSelectedObject)
+			(loop ticks)
+			(begin  (d-PopPosition) ticks)
+		)
+	)
+	#f ; no selection
+    )
+
+)
+
+(define* (duration::ChangeNoteDurationInTicks ticks #:optional (dots 0))
+; First change the base-duration. The d-Change commands also delete any dots.
+  (define (changeBase number) (case number
+   ;((12288) (d-ChangeMaxima))  ;Maxima
+   ((6144)	(d-ChangeLonga)) ; Longa
+   ((3072)	(d-ChangeBreve)) ; Breve
+   ((1536)	(d-Change0)) ; Whole
+   ((768)	(d-Change1)) ; Half
+   ((384)	(d-Change2)) ; Quarter
+   ((192)	(d-Change3)) ; Eighth
+   ((96)	(d-Change4)) ; Sixteenth
+   ((48)	(d-Change5)) ; 1/32
+   ((24)	(d-Change6)) ; 1/64
+   ((12)	(d-Change7)) ; 1/128
+   ((6)		(d-Change8)) ; 1/256
+   (else   #f )
+ ))
+ 
+ 
+; Second step: add dots
+  ; d-ChangeN work on appending position, but Breve and Longa not. But d-AddDot works on appending, too. So we must rule Appending out, else it will add dots without changing the duration for breve and longa.
+  (if (and (string-ci=? (d-GetType) "CHORD") (integer? ticks) (integer? dots) (changeBase ticks)) ; <-- the action changeBase itself needs to be a test, too. 
+  (let loop ((i 0))
+	(if (= dots i)
+	#t
+	(begin
+	   (d-AddDot)  
+	   (loop (+ i 1)))))
+  #f)
+)
+
+
+
+;;;;;;;;;;;;;;
+
+
+(define MIDI-shortcuts::alist '(("" . "")))
+(define (SetMidiShortcut shortcut command)
+	(set! MIDI-shortcuts::alist (assoc-set! MIDI-shortcuts::alist shortcut command)))
+
+(SetMidiShortcut "FootpedalUp" #f)
+(SetMidiShortcut "FootpedalDown" #f)
+
+(define (MIDI-shortcut::controller type value)
+;;(format #t "controller type ~a value ~a\n" type value)
+  (cond ((and (equal? type 64) (equal? value 127))
+    	  (assoc-ref MIDI-shortcuts::alist "FootpedalUp"))
+        ((and (equal? type 64) (equal? value 0))
+    	  (assoc-ref MIDI-shortcuts::alist "FootpedalDown"))
+
+        ((equal? type 1)
+	 (let ((thestep  (round(/ (- value 64) 16))))
+	 (PlayNote  
+	  (number->string  (+ 60 (* 4 thestep) ))
+	  100)
+	     
+	 (eval-string (string-append "(d-SetEnharmonicPosition " (number->string  thestep) ")"))
+         #f)
+        )
+
+
+      (else #f)))
+
+
+(define (MIDI-shortcut::pitchbend value)
+(format #t "pitch bend value ~a\n" value)
+  (cond ((> value 8192)
+    	  (assoc-ref MIDI-shortcuts::alist "PitchUp"))
+         ((< value 8192)
+    	  (assoc-ref MIDI-shortcuts::alist "PitchDown"))
+      (else #f)))
