@@ -2,25 +2,25 @@
  ; Get all notes on cursor position and create a list with new values which then exchanges the current notes on cursor position
 
 (if (d-GetNotes)
-    (ANS-7::ChangeChordNotes  
-  	(map method (ANS-7::GetChordNotes))
+    (ANS::ChangeChordNotes  
+  	(map method (ANS::GetChordNotes))
   	)
     #f ; not a note/chord
   )
 )
 
 (define (ShiftUp)
- (ShiftProto ANS-7::CalculateDiatonicStepUp)
+ (ShiftProto ANS::CalculateDiatonicStepUp)
 )
 
 (define (ShiftDown)
- (ShiftProto ANS-7::CalculateDiatonicStepDown)
+ (ShiftProto ANS::CalculateDiatonicStepDown)
 )
 
 (define (ShiftRealOctaveUp) ;in reality this is not shift but transpose. But there are too many functions with the name transpose already...
- (ShiftProto ANS-7::CalculateRealOctaveUp)
+ (ShiftProto ANS::CalculateRealOctaveUp)
 )
 
 (define (ShiftRealOctaveDown) ;in reality this is not shift but transpose. But there are too many functions with the name transpose already...
- (ShiftProto ANS-7::CalculateRealOctaveDown)
+ (ShiftProto ANS::CalculateRealOctaveDown)
 )
