@@ -52,6 +52,23 @@ typedef enum
     REDO
   }unre_mode;
 
+
+
+typedef enum DenemoGraphicType
+  {
+    DENEMO_BITMAP,
+    DENEMO_PATTERN
+  } DenemoGraphicType;
+
+typedef struct DenemoGraphic
+{
+  DenemoGraphicType type;
+  gpointer graphic; /**< either a GdkBitmap or a cairo_pattern_t */
+  gint width, height;
+}
+DenemoGraphic;
+
+
 /* The idea here is to make everything recursive.  The dominant
    paradigm is a linked list.  Handy that there's such a nice
    precooked implementation of them in glib, eh?  Note, however, that
@@ -159,8 +176,8 @@ typedef struct
   DenemoObjType type; /**< The type of object pointed to by the gpointer object field below */
   gchar *user_string;/**< Holds user's original text parsed to generated this 
 			object */
-  gint basic_durinticks;
-  gint durinticks; /**< Duration of object */
+  gint basic_durinticks; /**< Duration of object including dotting but not tuplet/grace note effects. */
+  gint durinticks; /**< Duration of object where 384 (PPQN) is a quarter note, includes any tuplet/grace note effects */
   gint starttick; /**< When the object occurs */ 
   gint starttickofnextnote; /**< When the next object occurs */
   GList* midi_events;/**< data are the smf_event_ts that this object gives rise to */
@@ -427,11 +444,11 @@ typedef struct DenemoDirective
   gint tx,ty; /**< x and y offsets in pixels for the display text */
   gint minpixels;/**< horizontal space needed by the display */
   gint x, y; /**< x and y offsets passed to LilyPond to control printed position */
-  GObject *graphic; /**< bitmap to draw for this directive */
+  DenemoGraphic *graphic; /**< what to draw for this directive */
   GtkWidget *widget;  /**<  a button or menu item for accessing the directive for editing or actioning */
   gint gx, gy; /**< x and y offsets in pixels for the graphic */
-  GString *graphic_name; /**< name of the graphic to be drawn */
-  gint width, height; /**< width and height of the bitmap */
+  GString *graphic_name; /**< name of the graphic to be drawn the suffixes .svg or .png will be searched for */
+
   /* warning these values cannot be changed without bumping the denemo file format version */
 #define DENEMO_OVERRIDE_LILYPOND (1<<0)
 #define DENEMO_OVERRIDE_UNUSED (1<<1)
@@ -445,8 +462,9 @@ typedef struct DenemoDirective
 #define DENEMO_OVERRIDE_REPEAT (1<<10)
 #define DENEMO_OVERRIDE_CHANNEL (1<<11)
 #define DENEMO_OVERRIDE_TEMPO (1<<12)
+#define DENEMO_OVERRIDE_TRANSPOSITION (1<<13)
 
-#define DENEMO_MIDI_MASK (DENEMO_OVERRIDE_VOLUME | DENEMO_OVERRIDE_DURATION | DENEMO_OVERRIDE_REPEAT | DENEMO_OVERRIDE_CHANNEL | DENEMO_OVERRIDE_TEMPO)
+#define DENEMO_MIDI_MASK (DENEMO_OVERRIDE_VOLUME | DENEMO_OVERRIDE_DURATION | DENEMO_OVERRIDE_REPEAT | DENEMO_OVERRIDE_CHANNEL | DENEMO_OVERRIDE_TEMPO | DENEMO_OVERRIDE_TRANSPOSITION)
 
 #define DENEMO_OVERRIDE_ONCE (1<<16)
 #define DENEMO_OVERRIDE_STEP (1<<17)
@@ -814,6 +832,7 @@ typedef struct DenemoGUI
 #define DENEMO_MAX_SYSTEMS (100) /**< Number of lines of music that can be displayed */
   gint lefts[DENEMO_MAX_SYSTEMS];/**< an array to hold the leftmeasurenum of each system in the last-drawn score, used for determining the mouse position on the music */
   gint rights[DENEMO_MAX_SYSTEMS];/**< an array to hold the rightmeasurenum of each system in the last-drawn score, used for determining the mouse position on the music */
+  gint scales[DENEMO_MAX_SYSTEMS];/**< an array to hold the percent horizontal scaling of each system in the last-drawn score, used for determining the mouse position on the music */
 
   GtkWidget *printarea;/**< area holding a print preview */
   GtkWidget *printvscrollbar;/**< scrollbar widget for printarea */
@@ -864,7 +883,8 @@ typedef struct DenemoGUI
   /* support for rhythm patterns */
   GList *rhythms;/**< list of RhythmPattern s */
   GList *currhythm; /**< currently in use element of rhythms */
-  GList *rstep; /**< step within RhythmPattern->rsteps, the current element of the current rhythm pattern */
+  GList *cstep; /**< step within RhythmPattern->clipboard, the current element of the current rhythm pattern represented by the clipboard */
+  GList *rstep; /**< OBSOLETE step within RhythmPattern->rsteps, the current element of the current rhythm pattern */
 
   struct RhythmPattern *prevailing_rhythm; /**< one of singleton_rhythms used for entering notes */
 }DenemoGUI;
@@ -879,6 +899,7 @@ typedef struct RhythmPattern
   GList *rsteps; /**< the data are RhythmElements */
   GtkToolButton *button; /**< the button on the rhythm toolbar which invokes this rhythm */
   GList *clipboard;/**< a Denemo clipboard, used to create this patttern. */
+  gchar *name;/**< a user-facing name for this pattern */
 } RhythmPattern;
 
 
