@@ -227,6 +227,7 @@ insertcomparefunc (gconstpointer a, gconstpointer b)
 
 /* modify the pitch of the note of a one note chord.
    (for multi-note chords delete and add a note, do not use this).
+FIXME repeated calls to calculateheight
 */
 void modify_note(chord *thechord, gint mid_c_offset, gint enshift, gint dclef) {
   note *thenote = (note*)(thechord->notes->data);
@@ -239,7 +240,11 @@ void modify_note(chord *thechord, gint mid_c_offset, gint enshift, gint dclef) {
     calculateheight (mid_c_offset, dclef);
   thechord->lowestpitch = mid_c_offset;
   thechord->lowesty =
-    calculateheight (mid_c_offset, dclef); 
+    calculateheight (mid_c_offset, dclef);
+  if(Denemo.gui->last_source==INPUTKEYBOARD) 
+     {DenemoStaff *curstaffstruct = (DenemoStaff *)  Denemo.gui->si->currentstaff->data;
+    playnotes (Denemo.prefs.immediateplayback, thechord, curstaffstruct->midi_channel);
+  }
 }
 
 /* Allocate a new note structure initializing the fields
@@ -455,8 +460,23 @@ shiftpitch (DenemoObject * thechord, gint mid_c_offset, gint is_sharpening)
 void
 changedur (DenemoObject * thechord, gint baseduration, gint numdots)
 {
+  gint current =  ((chord *) thechord->object)->baseduration;
+  if(current<0) {
+    if(((chord*)thechord->object)->directives) {
+      GList *g;
+      for(g=((chord*)thechord->object)->directives;g;g=g->next) {
+	DenemoDirective *directive = g->data;
+	if(directive->prefix) {
+	  free_directive(directive);
+	  ((chord*)thechord->object)->directives = g_list_remove(((chord*)thechord->object)->directives, directive);
+	  break;//there can only be one prefix replacing the duration, it would be tricky to remove more than one anyway as continuing the loop would be trick...
+	}	
+      }	
+    }  
+  } 
   ((chord *) thechord->object)->baseduration = baseduration;
   ((chord *) thechord->object)->numdots = numdots;
+  
   set_basic_numticks (thechord);
 }
 
