@@ -42,11 +42,16 @@ static ClefInfo clef_info[] = {
   {DENEMO_FRENCH_CLEF, N_("French")},
 };
 
+const gchar *get_clef_name(gint type) {
+  if(type>=0 && (type<G_N_ELEMENTS(clef_info)))
+    return  clef_info[type].name;
+  return NULL;
+}
 
 static enum clefs get_clef_from_name(gchar *name) {
   gint i;
   for(i=0;i<G_N_ELEMENTS(clef_info);i++) {
-    if(!strcmp(name, clef_info[i].name))
+    if(!g_ascii_strcasecmp(name, clef_info[i].name))
       return clef_info[i].clef;
   }
   return DENEMO_INVALID_CLEF;
