@@ -163,6 +163,7 @@ void
 toggle_end_diminuendo (DenemoGUI *gui);
 
 void insertion_point (DenemoScore  *si);
+void insertion_point_for_type (DenemoScore * si, DenemoObjType type);
 
 void
 caution (DenemoScore  *si);
@@ -188,6 +189,9 @@ void deletemeasure(DenemoGUI * gui);
 void deletemeasureallstaffs(DenemoGUI * gui);
 void deleteobject(DenemoGUI *gui);
 
+
+void insert_note_following_pattern(DenemoGUI *gui);
+
 void
 beamandstemdirhelper (DenemoScore * si);
 
