@@ -120,7 +120,8 @@ struct infotopass
   gint in_highy; // FIXME these are passed in so that highy, lowy do not need to be passed back
   gint in_lowy;
   gboolean mark;//whether the region is selected
-  gint *left, *right;//location of right and left measurenum for current system(line)
+  gint *left, *right;//pointer into array, pointing to leftmost/rightmost measurenum for current system(line)
+  gint *scale;//pointer into an array of scales - this entry is the percent horizontal scale applied to the current system
   GList *last_midi;//last list of midi events for object at right of window
   DenemoObject *startobj;//pointer values - if drawing such an object mark as playback start
   DenemoObject *endobj;//pointer values - if drawing such an object mark as playback end
@@ -575,10 +576,12 @@ draw_measure (cairo_t *cr, measurenode * curmeasure, gint x, gint y,
 
 
     if(cr) {
-    if(itp->measurenum >= si->rightmeasurenum+1)
-	cairo_set_source_rgb( cr, 0.5,0.5,0.5 );//This draws light gray anything that may be only partially visible.
-    else 
-      cairo_set_source_rgb( cr, 0, 0, 0 );//black;
+      //      if(itp->measurenum > si->rightmeasurenum+1)
+      //	g_critical("Please advise the circumstance in which this happened");
+      if(itp->measurenum == si->rightmeasurenum+1)
+	cairo_set_source_rgb( cr, 0.5,0.5,0.5 );//This draws light gray anything that will be repeated at the start of the next page.
+      else 
+	cairo_set_source_rgb( cr, 0, 0, 0 );//black;
     }// if cr
 
     extra_ticks = draw_object (cr, curobj, x, y, gui, itp);
@@ -653,8 +656,14 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
   DenemoScore *si = gui->si;
   gint x  = KEY_MARGIN, i;
   //g_print("drawing staff %d at %d\n", itp->staffnum, y);
+  gint nummeasures = g_list_length (thestaff->measures);
+
+  // g_print("Of %d current %d\n", nummeasures, itp->measurenum);
+  if(itp->measurenum>nummeasures)
+    cr=NULL;//no more drawing on this staff
   if(cr) {
 	  cairo_save(cr);
+	 
   //Draw vertical lines to bind the staffs of the system together
       if(curstaff->prev)
 	{
@@ -666,7 +675,7 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
       if(curstaff->next)
 	{
 	  DenemoStaff *next = (DenemoStaff *)(curstaff->next->data);
-	  cairo_save(cr);
+	  //cairo_save(cr);
 	  cairo_set_source_rgb( cr, 0, 0, 0);
 	  cairo_rectangle (cr, LEFT_MARGIN, y, 2, 2*STAFF_HEIGHT + next->space_above + thestaff->space_below);
 	  cairo_fill(cr);	 
@@ -702,7 +711,7 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
     x += SPACE_FOR_TIME;// to allow the same margin ??
   }
 
-  *itp->left = itp->measurenum;
+  *itp->left = itp->measurenum>gui->si->rightmeasurenum?gui->si->rightmeasurenum:itp->measurenum; 
   memcpy (itp->keyaccs, thestaff->leftmost_keysig->accs, SEVENGINTS);
 
 
@@ -727,7 +736,7 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
  cairo_restore(cr);
   }
 
-  cairo_save(cr);
+  // cairo_save(cr);
   }//if cr
 
   if((gui->si->smf) && (itp->startobj==NULL) && (itp->startposition<=0) && (si->leftmeasurenum==1))
@@ -757,24 +766,43 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
     //g_print("Drawing staff %d leftmost time %f, measurenum %d\n",itp->staffnum, itp->leftmosttime, itp->measurenum);
   }
 
+  if(!*itp->scale)
+    *itp->scale=100;
+ if(cr) {
+   cairo_scale(cr, 100.0/(*itp->scale),1.0);
+	    //cairo_scale(cr,(*itp->scale)/100.0,1.0);
+ }
 
+ gint scale_before=*itp->scale;
   itp->line_end = FALSE;
-  while ( (!itp->line_end)  //       itp->measurenum <= si->rightmeasurenum+1
-	 && itp->measurenum <= g_list_length (thestaff->measures))
+  
+  while ( (!itp->line_end) 
+	 && itp->measurenum <= nummeasures)
     {
       if( x + GPOINTER_TO_INT (itp->mwidthiterator->data) + SPACE_FOR_BARLINE >
 	  (int) (gui->scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME)))
-	if(itp->curmeasure->next) {
 	  itp->line_end=TRUE;
-	  // if(Denemo.gui->si->playingnow)
-	  //	continue;//don't show grayed out measures while playing.
-	}
+	 
       itp->last_gap = 0;
       draw_measure (cr, itp->curmeasure, x, y, gui, itp);
 
       x += GPOINTER_TO_INT (itp->mwidthiterator->data) + SPACE_FOR_BARLINE;
+
+#if 0
+      if(itp->line_end) {
+      *itp->scale = itp->curmeasure->next?
+	(int)(100*x/(gui->scorearea->allocation.width/gui->si->zoom)):100;
+      }
+#else
+      if(itp->line_end && itp->curmeasure->next)
+	*itp->scale = (int)(100*x/(gui->scorearea->allocation.width/gui->si->zoom));
+      if( (Denemo.gui->view!=DENEMO_PAGE_VIEW) && *itp->scale < 100)
+	*itp->scale = 100;
+#endif
+
       itp->curmeasure = itp->curmeasure->next;
       itp->mwidthiterator = itp->mwidthiterator->next;
+
       itp->measurenum++;
       //g_print("line_end is %d, while itp->measurenum=%d and si->rightmeasurenum=%d\n",  itp->line_end, itp->measurenum, si->rightmeasurenum);
       if(!itp->line_end) {
@@ -784,13 +812,14 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
 	}
 	if(itp->lowy>itp->in_lowy && itp->lowy<MAXEXTRASPACE){
 	  thestaff->space_below = itp->lowy;
-	  repeat=TRUE;
+	  repeat = TRUE;
 	}
       }			      
-    }
-
+    } // end of loop drawing each measure
+  if(scale_before!=*itp->scale)
+    repeat = TRUE;/* the first system is already drawn, so it is too late to discover that we needed to scale it */
   *itp->right = itp->measurenum-1;
-  
+  /* draw lines connecting the staves in this system at the left and right hand ends */
   if(cr) if(curstaff->prev)
 	{
 	  DenemoStaff *prev = (DenemoStaff *)(curstaff->prev->data);	  
@@ -801,27 +830,28 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
       if(cr) if(curstaff->next)
 	{
 	  DenemoStaff *next = (DenemoStaff *)(curstaff->next->data);
-	  cairo_save(cr);
+	  //cairo_save(cr);
 	  cairo_set_source_rgb( cr, 0, 0, 0);
 	  cairo_rectangle (cr, x - SPACE_FOR_BARLINE, y, 2, 2*STAFF_HEIGHT + next->space_above + thestaff->space_below);
 	  cairo_fill(cr);	 
 	}
-
+  /* end of draw lines connecting the staves in this system at the left and right hand ends */
 
   if(cr) cairo_restore(cr);
   // if(itp->highy > title_highy)
   //  itp->highy = title_highy;
 
   /* now draw the staff lines, reset itp->slur_stack, and we're done */
-  if(cr) for (i = 0; i < thestaff->no_of_lines; i++, y += LINE_SPACE) {
+  if(cr) {
     cairo_set_line_width( cr, 1.0 );
-    cairo_move_to( cr, LEFT_MARGIN, y );
-    cairo_line_to( cr, x - HALF_BARLINE_SPACE, y );
+    for (i = 1; i <= thestaff->no_of_lines; i++) {
+      gint yy = y+2*LINE_SPACE+((i%2)?(1):(-1))*(i/2)*LINE_SPACE;
+      cairo_move_to( cr, LEFT_MARGIN, yy );
+      cairo_line_to( cr, (x*100/(*itp->scale)) - HALF_BARLINE_SPACE, yy );    
+    }
     cairo_stroke( cr );
   }
-
- 
-
+  
   /* Initialize the slur_stack for this staff. For the time being,
      slurs that begin and/or end after the portion of the music
      that is shown are not drawn. */
@@ -830,7 +860,7 @@ draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
       g_slist_free (itp->slur_stack);
       itp->slur_stack = NULL;
     }
-  if(cr) cairo_restore(cr);
+  //if(cr) cairo_restore(cr);
   return repeat;
 }
 static void
@@ -1037,6 +1067,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
     itp.line_end = FALSE;
     itp.left = &gui->lefts[0];
     itp.right = &gui->rights[0];
+    itp.scale = &gui->scales[0];
 
 
 
@@ -1060,6 +1091,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
     yy = y + line_height;
     itp.left++;
     itp.right++;
+    itp.scale++;
     if(Denemo.gui->si->playingnow && itp.measurenum >= si->rightmeasurenum)
       itp.line_end = FALSE;//don't print whole lines of grayed out music during playback
 
@@ -1075,6 +1107,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
       yy += line_height;
       itp.left++;
       itp.right++;
+      itp.scale++;
 
     } //end while printing out all the systems for this staff
 
@@ -1086,6 +1119,7 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
       //put the next line of music at the top with a break marker
       itp.left = &gui->lefts[0];
       itp.right = &gui->rights[0];     
+      itp.scale = &gui->scales[0];     
 
 	cairo_save(cr);
 	if(0)   {
@@ -1196,7 +1230,7 @@ DenemoGUI *gui = Denemo.gui;
   }
 
   /* Draw the score */
-  } while(draw_score (widget, gui));
+  } while(draw_score (widget, gui) && !Denemo.gui->si->playingnow);
 
   /* Now actually draw the backing pixmap onto the drawing area */
 
