@@ -12,7 +12,6 @@
 #include "gcs.h"
 #include <math.h>
 
-
 gint restwidths[SMALLESTDURATION + 1] =
   { WHOLEREST_WIDTH, HALFREST_WIDTH, QUARTERREST_WIDTH, EIGHTHREST_WIDTH,
   SIXTEENTHREST_WIDTH, THIRTYSECONDREST_WIDTH, SIXTYFOURTHREST_WIDTH, HUNDREDTWENTYEIGHTHREST_WIDTH, TWOHUNDREDFIFTYSIXTHREST_WIDTH
@@ -20,6 +19,7 @@ gint restwidths[SMALLESTDURATION + 1] =
 gint headwidths[3] = { WHOLEHEAD_WIDTH, HALFHEAD_WIDTH, NOTEHEAD_WIDTH
 };
 
+
 /**
  * draw_dots
  * This draws dots after rests or notes 
@@ -44,9 +44,9 @@ draw_dots (cairo_t *cr,
  * This function actually draws a rest onto the backing pixmap 
  *
  */
-void
+static void
 draw_rest (cairo_t *cr,
-	   gint duration, gint numdots, gint xx, gint y)
+	   gint duration, gint numdots, gint xx, gint y, DenemoGraphic *override_rest, gint gx, gint gy)
 {
   static gint restoffsets[SMALLESTDURATION + 1] =
     { WHOLEREST_OFFSETFROMTOP, HALFREST_OFFSETFROMTOP,
@@ -58,10 +58,13 @@ draw_rest (cairo_t *cr,
   };
 
   static gunichar rest_char[SMALLESTDURATION + 1] =
-    { 0x20, 0x21, 0x27, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e
+    { 0x20, 0x21, 0x27, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2d/*note no glyph for this one! */ 
   };
-
-  drawfetachar_cr (cr, rest_char[duration],
+  if(override_rest)
+    drawbitmapinverse_cr ( cr, override_rest,
+			       xx+gx+ restwidths[0]-override_rest->width/2,  y+restoffsets[0]+gy-override_rest->height/2);
+  else
+    drawfetachar_cr (cr, rest_char[duration],
 		     xx, y + restoffsets[duration]);
   /* Now draw any trailing dots and we're done */
 
@@ -74,10 +77,10 @@ draw_rest (cairo_t *cr,
  *  This function actually draws the note onto the backing pixmap 
  *
  */
-void
+static void
 draw_notehead (cairo_t *cr,
 	       note * thenote, gint duration, gint numdots,
-	       gint xx, gint y, gint * accs, gint is_stemup)
+	       gint xx, gint y, gint * accs, gint is_stemup, gboolean invisible, DenemoGraphic *override_notehead, gint gx, gint gy)
 {
   /* Adam's changed this code; it used to be that these arrays only had
      three elements.  The change has defeated what had been semi-elegance;
@@ -112,7 +115,7 @@ draw_notehead (cairo_t *cr,
     {
       noteheadtype = 3;
     }
-
+  if(duration<0) noteheadtype = 0;
 
   /* Draw the accidental, if necessary.  Note that this has to be
      done before xx is modified, as the the value in
@@ -131,38 +134,51 @@ draw_notehead (cairo_t *cr,
     else
       xx -= headwidths[noteheadtype];
   }
+  //OVERRIDE_GRAPHIC just affects the head, everything else -  accidentals dots stems beams ...are controlled by baseduration
   if(!(get_override(thenote->directives)&DENEMO_OVERRIDE_GRAPHIC)) {
+    if(override_notehead) {
+      //g_print("drawing a chord override graphic at %d %d\n",  xx+gx-override_notehead->width/2,  y+height+gy-override_notehead->height/2);
+      drawbitmapinverse_cr ( cr, override_notehead,
+			       xx+gx-override_notehead->width/2,  y+height+gy-override_notehead->height/2);
+    }
+    else {
     if (is_stemup)
       drawfetachar_cr ( cr, head_char[noteheadtype], xx, y + height);
     else
       drawfetachar_cr ( cr, head_char[noteheadtype], xx-0.5, y + height);
-    /* Now draw any trailing dots */
-    if ((height % LINE_SPACE) == 0)
-      draw_dots (cr, xx + headwidths[noteheadtype],
-		 y + height - HALF_LINE_SPACE, numdots);
-    else
-      draw_dots (cr, xx + headwidths[noteheadtype],
-		 y + height, numdots);
+    }
   }
   
-  /* any display for attached LilyPond */
+  gint maxwidth = headwidths[noteheadtype];
+  
+  /* any display for note directives */
  { GList *g = thenote->directives;
- GString *gstr=g_string_new("");
  gint count=10;
   for(;g;g=g->next, count+=10) {
     DenemoDirective *directive = (DenemoDirective *)g->data;
     if(directive->graphic) {
-      gint width, height;
-      gdk_drawable_get_size(GDK_DRAWABLE(directive->graphic), &width, &height);
-      drawbitmapinverse_cr ( cr, (GdkBitmap *)directive->graphic,
-			 xx+directive->gx+count,  y+thenote->y+directive->gy/*thechord.highesty*/, width, height);
+      gint gwidth, gheight;
+      gwidth = directive->graphic->width;
+      gheight = directive->graphic->height;
 
+      maxwidth = MAX(gwidth, maxwidth);
+      //g_print("drawing a graphic at %d %d\n", xx+directive->gx+count-gwidth/2,  y+height+directive->gy-gheight/2);
+      drawbitmapinverse_cr ( cr, directive->graphic,
+			       xx+directive->gx+count-gwidth/2,  y+height+directive->gy-gheight/2);
+      
     }
     if(directive->display) {
       drawnormaltext_cr (cr, directive->display->str, xx+directive->tx+count, y+thenote->y+directive->ty ); 
     }
   }
  }
+ /* Now draw any trailing dots */
+ if ((height % LINE_SPACE) == 0)
+   draw_dots (cr, xx + maxwidth,
+	      y + height - HALF_LINE_SPACE, numdots);
+ else
+   draw_dots (cr, xx + maxwidth,
+	      y + height, numdots);
 }
 
 /**
@@ -233,13 +249,16 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
   gint noteheadtype = MIN (duration, 2);
 
   /* Change those two so that they're cached instead */
+  if(duration<0) noteheadtype = 0;
+
   gint i;
   gint beampainty, arcwidth;
 
   gint prevbaseduration, nextbaseduration;
   GList *curnode;
   gboolean is_grace = thechord.is_grace && thechord.notes;
-
+  DenemoGraphic *override_notehead = NULL;//overriding notehead to be used for all notes of chord unless built-in or overriden
+  gint gx=0, gy=0;//positioning for overriding notehead
   cairo_save(cr);
   cairo_set_line_width( cr, 1.0 );
 
@@ -249,7 +268,7 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
     cairo_scale( cr, 0.8, 0.8);
     cairo_translate( cr, -xx, -(y+thenote->y));
   }
-
+  //  g_print("Invisble is %d\n", mudelaitem->isinvisible); 
   if (mudelaitem->isinvisible) {
     if (selected)
       cairo_set_source_rgb( cr, 231.0/255 , 1, 39.0/255 ); 
@@ -257,22 +276,35 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
       cairo_set_source_rgb( cr, 180.0/255, 160.0/255, 32.0/255 );// yellow for non printing
   }
 
-
+  { GList *g = thechord.directives;
+      gint count = 0;
+      for(;g;g=g->next) {
+	DenemoDirective *directive = (DenemoDirective *)g->data;
+	if(directive->graphic) {
+	  if(directive->override&DENEMO_OVERRIDE_GRAPHIC)
+	    gx=directive->gx, gy=directive->gy, override_notehead = directive->graphic;//will be used to draw all the notes/the rest
+	  else
+	    drawbitmapinverse_cr (cr, directive->graphic,
+				  xx+directive->gx, y+directive->gy);
+	}
+	if(directive->display) {
+	  drawnormaltext_cr (cr, directive->display->str, xx+directive->tx, y+STAFF_HEIGHT+40+count+directive->ty );
+	  count += 16;
+	}
+      } //for each chord directive
+  }//block displaying chord directives
   if (!thechord.notes/* a rest */) {
-     if( !(get_override(thechord.directives)&DENEMO_OVERRIDE_GRAPHIC))
-       draw_rest (cr, duration, thechord.numdots, xx, y);  
+    draw_rest (cr, MAX(duration, 0), thechord.numdots, xx, y, override_notehead, gx, gy);
   }  else {
     /* Draw the noteheads and accidentals */
-    
     for (curnode = thechord.notes; curnode; curnode = curnode->next){
       note *thenote = (note *)curnode->data;
-
       draw_notehead (cr, thenote, duration,
-		     thechord.numdots, xx, y, accs, thechord.is_stemup);
-
+		     thechord.numdots, xx, y, accs, thechord.is_stemup, mudelaitem->isinvisible, override_notehead, gx, gy); 
     }
-    /* Now the stem and beams. This is complicated. */
-    
+  }
+  /* Now the stem and beams. This is complicated. */
+  if (thechord.notes/* not a rest */) {
     if (thechord.is_stemup)
       {
 	if (mudelaitem->isstart_beamgroup && mudelaitem->isend_beamgroup)
@@ -280,19 +312,19 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	    if (duration >= 3)
 	      /* Up-pointing stem pixmap */
 	      drawfetachar_cr (cr, upstem_char[duration],
-				 xx + NOTEHEAD_WIDTH,
-				 thechord.highesty + y + 3
-				 - (duration == 6 ? EXTRA_STEM_HEIGHT
-				    : STEM_HEIGHT));
+			       xx + NOTEHEAD_WIDTH,
+			       thechord.highesty + y + 3
+			       - (duration == 6 ? EXTRA_STEM_HEIGHT
+				  : STEM_HEIGHT));
 	  }
 	else if (nextmuditem && !mudelaitem->isend_beamgroup)
 	  {
 	    /* Draw the thin beam across the gap */
 	    cairo_rectangle (cr,
-				xx + headwidths[noteheadtype] - 1,
-				y + thechord.stemy,
-				nextmuditem->x - mudelaitem->x,
-				THINBEAM_HEIGHT);
+			     xx + headwidths[noteheadtype] - 1,
+			     y + thechord.stemy,
+			     nextmuditem->x - mudelaitem->x,
+			     THINBEAM_HEIGHT);
 	    cairo_fill(cr);
 	    if (mudelaitem->isstart_beamgroup || !prevmuditem)
 	      prevbaseduration = 0;
@@ -308,18 +340,18 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 		if (nextbaseduration >= i) {
 		  /* Draw a thick beam across the gap */
 		  cairo_rectangle (cr,
-				      xx + headwidths[noteheadtype] - 1,
-				      y + beampainty,
-				      nextmuditem->x - mudelaitem->x,
-				      THICKBEAM_HEIGHT);
+				   xx + headwidths[noteheadtype] - 1,
+				   y + beampainty,
+				   nextmuditem->x - mudelaitem->x,
+				   THICKBEAM_HEIGHT);
 		  cairo_fill(cr);
-
+		  
 		} else if (prevbaseduration < i) {
 		  /* Draw a stub to the right of the staff */
 		  cairo_rectangle (cr,
-				      xx + headwidths[noteheadtype] - 1,
-				      y + beampainty, STUB_WIDTH,
-				      THICKBEAM_HEIGHT);
+				   xx + headwidths[noteheadtype] - 1,
+				   y + beampainty, STUB_WIDTH,
+				   THICKBEAM_HEIGHT);
 		  cairo_fill( cr );
 		}
 	      }		/* end for loop */
@@ -336,9 +368,9 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 		{
 		  /* Draw a stub to the left of the staff */
 		  cairo_rectangle (cr,
-				      xx + headwidths[noteheadtype] - 1 -
-				      STUB_WIDTH, y + beampainty, STUB_WIDTH,
-				      THICKBEAM_HEIGHT);
+				   xx + headwidths[noteheadtype] - 1 -
+				   STUB_WIDTH, y + beampainty, STUB_WIDTH,
+				   THICKBEAM_HEIGHT);
 		  cairo_fill(cr);
 		}
 	  }
@@ -349,7 +381,7 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	  cairo_line_to( cr, xx + headwidths[noteheadtype], thechord.lowesty + y - 2 );
 	  cairo_stroke( cr );
 	}
-
+	
 	
 	/* Now draw the tie, if appropriate */
 	if (thechord.is_tied)
@@ -358,7 +390,7 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	      arcwidth = nextmuditem->x - mudelaitem->x;
 	    else
 	      arcwidth = mwidth - mudelaitem->x + SPACE_FOR_BARLINE;
-
+	    
 	    cairo_set_line_width( cr, 1.0 );
 	    cairo_move_to( cr, xx + headwidths[noteheadtype] / 2, y + thechord.highesty - 13 );
 	    cairo_rel_curve_to( cr, arcwidth/3, -8, arcwidth*2/3, -8, arcwidth, 0 );
@@ -372,11 +404,11 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	    if (duration >= 3)
 	      /* Down-pointing stem */
 	      drawfetachar_cr (cr, downstem_char[duration],
-				 xx,
-				 thechord.lowesty + y 
-				 + (duration == 6 ? EXTRA_STEM_HEIGHT
-				    : STEM_HEIGHT)
-				 );
+			       xx,
+			       thechord.lowesty + y 
+			       + (duration == 6 ? EXTRA_STEM_HEIGHT
+				  : STEM_HEIGHT)
+			       );
 	  }
 	else if ((nextmuditem) && !mudelaitem->isend_beamgroup)
 	  {
@@ -385,7 +417,7 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 			     nextmuditem->x - mudelaitem->x,
 			     THINBEAM_HEIGHT);
 	    cairo_fill( cr );
-
+	    
 	    if (mudelaitem->isstart_beamgroup || !prevmuditem)
 	      prevbaseduration = 0;
 	    else
@@ -402,14 +434,14 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 		if (nextbaseduration >= i) {
 		  /* Draw a thick beam across the gap */
 		  cairo_rectangle( cr, xx, y + beampainty,
-		      nextmuditem->x - mudelaitem->x,
-		      THICKBEAM_HEIGHT);
+				   nextmuditem->x - mudelaitem->x,
+				   THICKBEAM_HEIGHT);
 		  cairo_fill( cr );
-
+		  
 		} else if (prevbaseduration < i) {
 		  /* Draw a stub to the right of the staff */
 		  cairo_rectangle (cr, xx, y + beampainty,
-				      STUB_WIDTH, THICKBEAM_HEIGHT);
+				   STUB_WIDTH, THICKBEAM_HEIGHT);
 		  cairo_fill(cr);
 		}
 	      }		/* End for loop */
@@ -417,20 +449,20 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	else
 	  {			/* We're at the end of a beamgroup */
 	    if(prevmuditem)//sanity check
-	    for (i = MAX (((chord *) prevmuditem->object)->baseduration,
-			  4),
-		   beampainty = thechord.stemy - FIRSTBEAMSPACE - 
-		   THICKBEAM_HEIGHT + 1 -
-		   (SUBSQBEAMSPACE * (i - 4));
-		 i <= thechord.baseduration;
-		 i++, beampainty -= SUBSQBEAMSPACE)
-	      {
-		/* Draw a stub to the left of the staff */
-		cairo_rectangle (cr, xx - STUB_WIDTH,
-				    y + beampainty, STUB_WIDTH,
-				    THICKBEAM_HEIGHT);
-		cairo_fill(cr);
-	      }
+	      for (i = MAX (((chord *) prevmuditem->object)->baseduration,
+			    4),
+		     beampainty = thechord.stemy - FIRSTBEAMSPACE - 
+		     THICKBEAM_HEIGHT + 1 -
+		     (SUBSQBEAMSPACE * (i - 4));
+		   i <= thechord.baseduration;
+		   i++, beampainty -= SUBSQBEAMSPACE)
+		{
+		  /* Draw a stub to the left of the staff */
+		  cairo_rectangle (cr, xx - STUB_WIDTH,
+				   y + beampainty, STUB_WIDTH,
+				   THICKBEAM_HEIGHT);
+		  cairo_fill(cr);
+		}
 	  }
 	
 	if (duration > 0) {
@@ -452,32 +484,17 @@ draw_chord ( cairo_t *cr, objnode * curobj, gint xx, gint y,
 	    cairo_stroke( cr );
 	  }
       }
-    /* End stemdown stuff */
-    
-    draw_articulations (cr, thechord, xx, y); 
-    if(is_grace) cairo_restore(cr);
-    draw_ledgers (cr, thechord.highesty, thechord.lowesty, xx, y,
-		  headwidths[noteheadtype]);
-    if(is_grace) cairo_save(cr);
-    
-  }				/* end else if there are notes in the chord*/
-  { GList *g = thechord.directives;
-  gint count = 0;
-  for(;g;g=g->next) {
-    DenemoDirective *directive = (DenemoDirective *)g->data;
-    if(directive->graphic) {
-      gint width, height;
-      gdk_drawable_get_size(GDK_DRAWABLE(directive->graphic), &width, &height);
-      drawbitmapinverse_cr (cr, (GdkBitmap*)directive->graphic,
-			 xx+directive->gx, y+directive->gy, width, height);
-    }
-    if(directive->display) {
-      drawnormaltext_cr (cr, directive->display->str, xx+directive->tx, y+STAFF_HEIGHT+40+count+directive->ty );
-      count += 16;
-    }
-  } //for each chord directive
-  }//block displaying chord directives
-  cairo_restore (cr);
+  /* End stemdown stuff */
+  
+  draw_articulations (cr, thechord, xx, y); 
+  if(is_grace) cairo_restore(cr);
+  draw_ledgers (cr, thechord.highesty, thechord.lowesty, xx, y,
+		headwidths[noteheadtype]);
+  if(is_grace) cairo_save(cr);
+  
+  }				/* end if not a rest draw stems etc */
+
+cairo_restore (cr);
 }
 
 
