@@ -49,7 +49,7 @@
 
 
 #define TAB "        "
-
+static gchar *get_postfix(GList *g);
 static void 
 create_lilywindow(DenemoGUI *gui);
 static void
@@ -303,6 +303,8 @@ determineclef (gint type, gchar ** clefname)
 static gint
 internaltomuduration (gint internalduration)
 {
+  if(internalduration < 0)
+    return internalduration;
   return 1 << internalduration;
 }
 
@@ -322,63 +324,13 @@ append_duration (GString * figures, gint duration, gint numdots)
     figures = g_string_append (figures, ".");
 }
 
-#if 0
-static void output_lyric(GString *lyrics, chord *pchord, gboolean *pis_syllable, gboolean *pcenter_lyric) {
-  /*Lyrics */
-		 
-  gint duration = internaltomuduration (pchord->baseduration);
-  if (pchord->lyric)
-    {	 
-      if (*pis_syllable && !pchord->is_syllable)
-	lyrics = g_string_append (lyrics, " ");
-      lyrics = g_string_append (lyrics, pchord->lyric->str);
-		  
-      if (pchord->center_lyric)
-	{
-	  lyrics = g_string_append (lyrics, "-- ");
-	  *pcenter_lyric = TRUE;// FIXME unused
-	}
-      else
-	*pcenter_lyric = FALSE;
-		  
-      if (pchord->is_syllable)
-	{
-	  *pis_syllable = TRUE;
-	  lyrics = g_string_append (lyrics, "__ ");
-	}
-      else
-	{
-	  *pis_syllable = FALSE;
-	  lyrics = g_string_append (lyrics, " ");
-	}
-    }
-  else if (*pis_syllable)
-    {
-      gint extend;
-      //g_print ("duration %d\t mod %d\n", duration, duration % 4);
-      if (duration < 4)
-	{
-	  for (extend = 0; extend < duration % 4; extend++)
-	    lyrics = g_string_append (lyrics, "__ ");
-	}
-      else if (duration > 4)
-	{
-	  for (extend = 0; extend < 4 % duration; extend++)
-	    lyrics = g_string_append (lyrics, "_");
-	}
-      else
-	lyrics = g_string_append (lyrics, "__ ");
-    } else
-      lyrics = g_string_append (lyrics, " _ ");//FIXME is this right?
-}
-#endif
 
 
 /**
  * add figures to *pfigures for *pchord  
  */
 static void
-output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
+output_figured_bass (DenemoScore * si, GString *figures, chord * pchord, gint time1, gint time2)
 {
   static gboolean continuation = FALSE;
   gboolean continuation_finishing = FALSE;
@@ -388,6 +340,25 @@ output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
   GString *fig_str;		/*working copy of figures string stored in pchord */
   char *str;			/* pointer into the figure string fig_str */
   gint num_groups = 1;		/* number of groups of figures */
+  gchar *duration_string=NULL;//whole measure rests etc
+  if(duration<0) {
+      gchar *lily = get_postfix(pchord->directives);
+      if(lily) {
+	duration_string = g_strrstr(lily, "R1*");
+	if(!duration_string) {
+	  g_warning("duration is special but cannot find R1* - output in figured bass");
+	  duration_string = g_strdup("R1*4/4");
+	}
+      }
+      else {
+	g_warning("duration is special but no directives to account for it - output in figured bass");
+	duration_string = g_strdup("R1*4/4");
+      }
+      duration_string++;//the duration, after the  R
+  }
+#define APPEND_DUR(a, b, c) duration<0?g_string_append(figures, duration_string):append_duration(a,b,c)
+
+
   if(!last_figure)
     last_figure = g_string_new("");
 
@@ -437,7 +408,7 @@ output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
 	g_string_assign(last_figure, fig_str->str);
       }
       figures = g_string_append (figures, ">");
-      append_duration (figures, duration, numdots);
+      APPEND_DUR (figures, duration, numdots);
       if(continuation_finishing) {
 	figures = g_string_append (figures, "\\set useBassFigureExtenders = ##f ");
 	continuation = FALSE;
@@ -460,12 +431,12 @@ output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
 	str = strtok (fig_str->str, FIGURES_SEP);
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, first_duration, 0);
+	APPEND_DUR (figures, first_duration, 0);
 	figures = g_string_append (figures, "<");
 	str = strtok (NULL, FIGURES_SEP);
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, second_duration, 0);
+	APPEND_DUR (figures, second_duration, 0);
       }
       break;
     case 3:
@@ -489,17 +460,17 @@ output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
 	str = strtok (fig_str->str, FIGURES_SEP);
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, first_duration, 0);
+	APPEND_DUR (figures, first_duration, 0);
 	figures = g_string_append (figures, "<");
 	str = strtok (NULL, FIGURES_SEP);
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, second_duration, 0);
+	APPEND_DUR (figures, second_duration, 0);
 	str = strtok (NULL, FIGURES_SEP);
 	figures = g_string_append (figures, "<");
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, third_duration, 0);
+	APPEND_DUR (figures, third_duration, 0);
       }
       break;
     case 4:
@@ -525,25 +496,26 @@ output_figured_bass (DenemoScore * si, GString *figures, chord * pchord)
 	str = strtok (fig_str->str, FIGURES_SEP);
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, first_duration, 0);
+	APPEND_DUR (figures, first_duration, 0);
 	figures = g_string_append (figures, "<");
 	str = strtok (NULL, FIGURES_SEP);
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, second_duration, 0);
+	APPEND_DUR (figures, second_duration, 0);
 	str = strtok (NULL, FIGURES_SEP);
 	figures = g_string_append (figures, "<");
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, third_duration, 0);
+	APPEND_DUR (figures, third_duration, 0);
 	str = strtok (NULL, FIGURES_SEP);
 	figures = g_string_append (figures, "<");
 	figures = g_string_append (figures, str);
 	figures = g_string_append (figures, ">");
-	append_duration (figures, fourth_duration, 0);
+	APPEND_DUR (figures, fourth_duration, 0);
       }
       break;
     }
+  // if(duration_string) g_free(--duration_string);
 }
 
 /**
@@ -748,7 +720,18 @@ return g_string_free(ret, FALSE);
 }
 
 
-
+/* insert editable prefix string from passed directives, updating duration and open brace count */
+static void directives_insert_prefix_editable(GList *directives, gint *popen_braces, gint *pprevduration, GtkTextIter *iter, gchar* invisibility, DenemoGUI *gui) {
+  GList *g = directives;
+  for(;g;g=g->next) {
+    DenemoDirective *directive = (DenemoDirective *)g->data;
+    if(directive->prefix && directive->prefix->len) {
+      *pprevduration = -1;
+      *popen_braces += brace_count(directive->prefix->str); 
+      insert_editable(&directive->prefix, directive->prefix->str, iter, invisibility, gui);
+    }
+  }
+}
 
 
 
@@ -806,6 +789,7 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
       case CHORD:
 
 	pchord = (chord *) curobj->object;
+	gchar *chord_prefix = get_prefix(pchord->directives);
 	duration = internaltomuduration (pchord->baseduration);
 	numdots = pchord->numdots;
 	is_chordmode = FALSE;
@@ -820,26 +804,38 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 		g_string_append_printf (fakechords, "\\grace {");
 	    } 
 	    //	  }
-	GList *g = pchord->directives;
-	for(;g;g=g->next) {
-	  DenemoDirective *directive = (DenemoDirective *)g->data;
-	  if(directive->prefix && directive->prefix->len) {
-	    prevduration = -1;
-	    open_braces += brace_count(directive->prefix->str); 
-	    insert_editable(&directive->prefix, directive->prefix->str, iter, invisibility, gui);
-	  }
-	}
-	    
+
+#if 0
+	    //now before duration
+
+	    static void directives_insert_prefx_editable(GList *directives, gint *popen_braces, gint *pprevduration, GtkIter *iter, gchar* invisibility, DenemoGUI *gui) {
+	      GList *g = directives;
+	      for(;g;g=g->next) {
+		DenemoDirective *directive = (DenemoDirective *)g->data;
+		if(directive->prefix && directive->prefix->len) {
+		  *pprevduration = -1;
+		  *popen_braces += brace_count(directive->prefix->str); 
+		  insert_editable(&directive->prefix, directive->prefix->str, iter, invisibility, gui);
+		}
+	      }
+	    }
+#endif	    
+
+   if(!get_lily_override (pchord->directives)) { //skip all LilyPond output for this chord
 	if (!pchord->notes)
 	  {			/* A rest */
+
 	    if (!curobj->isinvisible)
 	      {
 		g_string_append_printf (ret, "r");
 		/* Duplicated code follows. I ought to fix that */
-		if (duration != prevduration || numdots != prevnumdots)
+		outputret;
+		directives_insert_prefix_editable(pchord->directives, &open_braces, &prevduration, iter, invisibility, gui);
+		if (duration != prevduration || numdots != prevnumdots || duration<0)
 		  {
 		    /* only in this case do we explicitly note the duration */
-		    g_string_append_printf (ret, "%d", duration);
+		    if(duration>0)
+		      g_string_append_printf (ret, "%d", duration);  
 		    prevduration = duration;
 		    prevnumdots = numdots;
 		    for (j = 0; j < numdots; j++)
@@ -849,8 +845,10 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 	    else
 	      {
 		g_string_append_printf (ret, "\\skip ");
-	
-		g_string_append_printf (ret, "%d", duration);
+		outputret;
+		directives_insert_prefix_editable(pchord->directives, &open_braces, &prevduration, iter, invisibility, gui);
+		  if(duration>0)
+		    g_string_append_printf (ret, "%d", duration);
 		prevduration = -1;
 		prevnumdots = -1;
 		for (j = 0; j < numdots; j++)
@@ -875,6 +873,8 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 		  {
 		    note *curnote = (note *) notenode->data;
 		    noteheadtype = curnote->noteheadtype;
+
+		    //As with chord-prefix, this is perhaps not a useful position, but until some other use is found for this field it is here...
 		    GList *g = curnote->directives;
 		    for(;g;g=g->next) {
 		      DenemoDirective *directive = (DenemoDirective *)g->data;
@@ -883,6 +883,8 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 			insert_editable(&directive->prefix, directive->prefix->len?directive->prefix->str:" ", iter, invisibility, gui);
 		      }
 		    }
+
+	if(!get_lily_override (curnote->directives)) { //skip all LilyPond output for this note
 		    switch (noteheadtype)
 		      {
 		      case DENEMO_NORMAL_NOTEHEAD:
@@ -950,6 +952,7 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 			output(" ");
 		   
 		    }
+	} /* End of LilyPond output for note, skipped if override set */
 		  }		/* End notes in chord loop */
 		
 		if (pchord->notes->next  || pchord->chordize) //multi-note chord
@@ -961,15 +964,21 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 		g_string_append_printf (ret, "s");
 		
 	      }
-	    if (duration != prevduration || numdots != prevnumdots)
+	    if (duration != prevduration || numdots != prevnumdots || duration<0)
 	      {
 		/* only in this case do we explicitly note the duration */
-		g_string_append_printf (ret, "%d", duration);
+		outputret;
+		directives_insert_prefix_editable(pchord->directives, &open_braces, &prevduration, iter, invisibility, gui);
+		  if(duration>0)
+		    g_string_append_printf (ret, "%d", duration);
 		prevduration = duration;
 		prevnumdots = numdots;
 		for (j = 0; j < numdots; j++)
 		  g_string_append_printf (ret, ".");
 	      }
+
+
+
 	    if (pchord->dynamics && (pchord->notes->next==NULL))
 	      {
 		dynamic_string = (GString *) pchord->dynamics->data;
@@ -1066,8 +1075,9 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 
 	    /* do this in caller                    g_string_append_printf (ret, " "); */
 	  } /* End of else chord with note(s) */
+   } /* End of skipping LilyPond for this chord because of LILYPOND_OVERRIDE set */
 
-	g = pchord->directives;
+	GList *g = pchord->directives;
 	for(;g;g=g->next) {
 	  DenemoDirective *directive = (DenemoDirective *)g->data;
 	  if(directive->postfix && directive->postfix->len) {
@@ -1086,6 +1096,7 @@ generate_lily_for_obj (DenemoGUI *gui, GtkTextIter *iter, gchar *invisibility, D
 	    g_string_append_printf (fakechords, "}");
 	}
 	
+	g_free(chord_prefix);
 
 
 	break;
@@ -1544,7 +1555,8 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 
 	  if(curobjnode) {
 	  curobj = (DenemoObject *) curobjnode->data;
-	  if (curobj->type==CHORD||curobj->type==PARTIAL||curobj->type==LILYDIRECTIVE)
+	  // if (curobj->type==CHORD||curobj->type==PARTIAL||curobj->type==LILYDIRECTIVE)
+	  if(curobj->durinticks||curobj->type==LILYDIRECTIVE)
 	    empty_measure=FALSE; 
 
 
@@ -1642,7 +1654,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 	      if (curmeasure->next)
 		g_string_append_printf(endstr,"%s", "\\Barline\n");
 	      else
-		g_string_append_printf(endstr, "%s"," \\bar \"|.\"\n");
+		g_string_append_printf(endstr, "%s"," \\EndMovementBarline\n");
 	    }
 	    
 	    gtk_text_buffer_get_iter_at_mark (gui->textbuffer, &iter, curmark);
@@ -1658,7 +1670,7 @@ outputStaff (DenemoGUI *gui, DenemoScore * si, DenemoStaff * curstaffstruct,
 
 	       
 	      if(curstaffstruct->hasfigures)
-		output_figured_bass (si, figures, pchord);
+		output_figured_bass (si, figures, pchord, cur_stime1, cur_stime2);
 	      
 	      if (curstaffstruct->hasfakechords)
 		output_fakechord(si, fakechords, pchord);
@@ -2083,7 +2095,7 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
 
     {//Score prefix
       //Default value for barline = barline check
-      gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter,  "\nBarline = |\n", -1, INEDITABLE, NULL, NULL);
+      gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter,  "\nBarline = |\nEndMovementBarline = \\bar \"|.\"\n", -1, INEDITABLE, NULL, NULL);
   GList *g = gui->lilycontrol.directives;
   for(;g;g=g->next) {
     DenemoDirective *directive = g->data;
@@ -2141,10 +2153,10 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
 	}  
       }
       /* the score prefix field is at the start of the whole score, the postfix field is placed as
-	 a prolog before the music in the scoreblock, unless the DENEMO_OVERRIDE_LILYPOND flag is set, in which case
-	 the postfix field replaces the scoreblock altogether */
+	 a prolog before the music in each movements scoreblock */
       gchar *score_prolog = get_postfix(gui->lilycontrol.directives);
-      g_string_append_printf (scoreblock, "%s", "\\score {\n");
+      gchar *movement_prolog = get_postfix(si->movementcontrol.directives);
+      g_string_append_printf (scoreblock, "%s", get_lily_override(si->movementcontrol.directives)? movement_prolog:"\\score {\n");
       g_string_append_printf (scoreblock, "<<%s <<\n", score_prolog);
       g_free(score_prolog);
     }
@@ -2291,62 +2303,28 @@ output_score_to_buffer (DenemoGUI *gui, gboolean all_movements, gchar * partname
 
       g_string_append_printf(scoreblock, "%s", ">>\n>>\n");
      
-      { gchar *layout_string = get_postfix(si->layout.directives);
-      g_string_append_printf(scoreblock, "%s", "\\layout{\n");
-      if(layout_string) {
-	g_string_append(scoreblock, layout_string);
-	g_free(layout_string);
-      }
-      g_string_append_printf(scoreblock, "%s", ""TAB"}\n");   
-      }
-
-
-
-
-      { gchar *header_string = get_postfix(si->header.directives);
-      //g_print("Now output %s for %p\n", header_string, si);
-      g_string_append_printf(scoreblock, "%s", "\\header{\n");
-      if(header_string) {
-	g_string_append(scoreblock, header_string);
-	g_free(header_string);
-      }  
-      g_string_append_printf(scoreblock, "%s", ""TAB"}\n\n"); /*end of  \header block */
-      }
+      if(!get_lily_override(si->layout.directives))
+	{ gchar *layout_string = get_postfix(si->layout.directives);
+	  g_string_append_printf(scoreblock, "%s%s%s", "\\layout {\n", layout_string?layout_string:"", ""TAB"}\n");
+	  g_free(layout_string);
+	}
+	
+	if(!get_lily_override(si->header.directives))
+	  { gchar *header_string = get_postfix(si->header.directives);	   
+	    g_string_append_printf(scoreblock, "%s%s%s", "\\header {\n", header_string?header_string:"", ""TAB"}\n");
+	    g_free(header_string);
+	  }
+	  
 
      
       g_string_append_printf(scoreblock, "%s", "}\n"); /*end of \score block */
-      /* any markup after score block*/
-      { 
-	gchar *mvmnt_string = get_postfix(si->movementcontrol.directives);
-	if(mvmnt_string) {
-	  g_string_append(scoreblock, mvmnt_string);
-	  g_free(mvmnt_string);
-	}
-      }
-      
+     
 
       // Put this standard scoreblock in the textbuffer
 
       gtk_text_buffer_get_iter_at_mark(gui->textbuffer, &iter, gtk_text_buffer_get_mark(gui->textbuffer, STANDARD_SCOREBLOCK));
 
-#if 0
-      if(get_lily_override(gui->lilycontrol.directives)) {
-	//FIXME this is being emitted for each movement, rather than just once
-	gchar *score_substitute = get_postfix(gui->lilycontrol.directives);
-	gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter, score_substitute, -1, INEDITABLE, NULL);
-	g_free(score_substitute);
-      }
-#else
-      if(get_lily_override(si->movementcontrol.directives)) {
-	gchar *scoreblock_substitute = get_postfix(si->movementcontrol.directives);
-	gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter, scoreblock_substitute, -1, INEDITABLE, NULL);
-	g_free(scoreblock_substitute);
-      }
-
-#endif
-
-
-      else    
+    
 	gtk_text_buffer_insert_with_tags_by_name (gui->textbuffer, &iter,scoreblock->str, -1, INEDITABLE,
 						  (partname==NULL && gui->custom_scoreblocks)?"invisible":NULL, NULL);
       
