@@ -1098,6 +1098,8 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
   int cur_volume;
   gboolean mute_volume;
 
+  int cur_transposition;
+
   int midi_channel = (-1);
   int tracknumber = 0;
   int timesigupper = 4;
@@ -1239,6 +1241,9 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 	g_warning("volume set to zero but not muted\nResetting volume\n");
 	cur_volume = 65;
       }
+      cur_transposition = curstaffstruct->transposition;
+
+
       if(tuplet>0)
 	g_print("Unterminated tuplet at end of voice %d\n", tracknumber-1);
       //Reset to  tuplets nesting level 0, in case unbalanced tuplet start end in last staff
@@ -1381,6 +1386,10 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			    case DENEMO_OVERRIDE_VOLUME:
 				change_volume(&cur_volume, midi_val, midi_interpretation, midi_action);
 			      break;
+			    case DENEMO_OVERRIDE_TRANSPOSITION:
+				cur_transposition = midi_val;
+			      break;
+
 			    case DENEMO_OVERRIDE_CHANNEL:
 				change_channel(&midi_channel, midi_val, midi_interpretation, midi_action);
 			      break;
@@ -1412,31 +1421,37 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 
 		  numdots = chordval.numdots;
 		  duration = 0;
-		  for (d = 0; d <= numdots; d++)
-		    {
-		      duration +=
-			internaltoticks (chordval.baseduration) >> d;
-		    }
-		  if (tuplet >= 1)
-		    {
-		      duration *= tupletnums.numerator;
-		      duration /= tupletnums.denominator;
-		      if (MIDI_RESOLUTION % tupletnums.denominator)
-			{
-			  measure_has_odd_tuplet = 1;
-			}
-		    }
+		  if(chordval.baseduration>=0) {
+		    for (d = 0; d <= numdots; d++)
+		      {
 
-		  if (tuplet >= 2)
-		    {
-		      if (MIDI_RESOLUTION % tupletnums.denominator *
-			  savedtuplet.denominator)
-			{
-			  measure_has_odd_tuplet = 1;
-			}
-		      duration *= savedtuplet.numerator;
-		      duration /= savedtuplet.denominator;
-		    }
+			
+			duration +=
+			  internaltoticks (chordval.baseduration) >> d;
+		      }
+		    if (tuplet >= 1)
+		      {
+			duration *= tupletnums.numerator;
+			duration /= tupletnums.denominator;
+			if (MIDI_RESOLUTION % tupletnums.denominator)
+			  {
+			    measure_has_odd_tuplet = 1;
+			  }
+		      }
+
+		    if (tuplet >= 2)
+		      {
+			if (MIDI_RESOLUTION % tupletnums.denominator *
+			    savedtuplet.denominator)
+			  {
+			    measure_has_odd_tuplet = 1;
+			  }
+			duration *= savedtuplet.numerator;
+			duration /= savedtuplet.denominator;
+		      }
+		  }
+		  else 
+		    duration = curobj->durinticks;
 
 	  /********************************
 	   * compute real duration of note
@@ -1528,7 +1543,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			    dia_to_midinote (mid_c_offset) + enshift;
 			  //printf ("transposition = %i\n",
 			  //	  curstaffstruct->transposition);
-			  notenumber += curstaffstruct->transposition;
+			  notenumber += cur_transposition;
 			  //printf ("notenumber = %i\n", notenumber);
 			  if(notenumber>127) {
 			    g_warning("Note out of range\n", notenumber = 60);
@@ -1809,6 +1824,10 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		      case  DENEMO_OVERRIDE_VOLUME:
 			change_volume(&cur_volume, midi_val, midi_interpretation, midi_action);
 			break;
+		      case DENEMO_OVERRIDE_TRANSPOSITION:
+			cur_transposition = midi_val;
+			break;
+
 		      case  DENEMO_OVERRIDE_CHANNEL:
 			change_channel(&midi_channel, midi_val, midi_interpretation, midi_action);
 			break;
@@ -1832,7 +1851,13 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			break;
 		      }
 		  }
-		  curobj->earliest_time = curobj->latest_time  = event->time_seconds;// can be the last event
+
+		 
+
+		  curobj->earliest_time   = event->time_seconds;// taking the last one...
+		  curobj->latest_time = curobj->earliest_time + curobj->durinticks*60.0/(cur_tempo*MIDI_RESOLUTION);
+		  ticks_read += curobj->durinticks;
+
 		  break;
 		default:
 #if DEBUG
