@@ -13,7 +13,7 @@
 #include "lyric.h"
 #include "lilydirectives.h"
 #include "texteditors.h"
-
+#include "xmldefs.h"
 #include <stdlib.h>
 #include <string.h>
 
@@ -229,8 +229,12 @@ determineDuration (gint duration, gchar ** durationName)
       *durationName = "one-hundred-twenty-eighth";
       break;
     default:
-      g_warning ("Unknown note duration 1/%d, using quarter", 1 << duration);
-      *durationName = "quarter";
+      if(duration<0)
+	*durationName = "whole";
+      else {
+	g_warning ("Unknown note duration 1/%d, using quarter", 1 << duration);
+	*durationName = "quarter";
+      }
       break;
     }
 }
@@ -609,7 +613,9 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
   //gint numerator, denominator;
   gchar *durationType;
   gdouble fraction = 0.0;
-
+  static gchar * version_string;
+  if(version_string==NULL)
+    version_string = g_strdup_printf("%d", CURRENT_XML_VERSION);
   /* Append .denemo onto the filename if necessary. */
 
   //if (strcmp (filename->str + filename->len - 7, ".denemo"))
@@ -622,11 +628,11 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
   /* Create the XML document and output the root element. */
   
   doc = xmlNewDoc ((xmlChar *) "1.0");
-  xmlSetDocCompressMode (doc, XML_COMPRESSION_RATIO);
+  //xmlSetDocCompressMode (doc, XML_COMPRESSION_RATIO);
   doc->xmlRootNode = scoreElem =
     xmlNewDocNode (doc, NULL, (xmlChar *) "score", NULL);
   ns = xmlNewNs (doc->xmlRootNode, (xmlChar *) DENEMO_XML_NAMESPACE, NULL);
-  xmlSetProp (scoreElem, (xmlChar *) "version", (xmlChar *) "2.0");
+  xmlSetProp (scoreElem, (xmlChar *) "version", (xmlChar *) version_string);
   /* FIXME: Put comment here ("Denemo XML file generated by..."). */
   
   if(getNumCharsSchemeText()) {
@@ -1060,6 +1066,9 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 			    newXMLDecoration (parentElem, ns, "arpeggio");
 			}
 		    }
+		  if(thechord->baseduration<0)
+		    newXMLIntChild (objElem, ns, (xmlChar *) "ticks", -thechord->baseduration);
+
 
 		  /*
 		   *  Output Dynamic which is now part of note 
@@ -1376,9 +1385,9 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 		      xmlNewChild (measureElem, ns, (xmlChar *) "lily-directive",
 				   (xmlChar *)" ");
 
-		  
-		  xmlSetProp (objElem, (xmlChar *) "locked",
-			      (xmlChar *) (((lilydirective *) curObj->object)->locked?"true":"false"));
+		  if(((lilydirective *) curObj->object)->locked)
+		    xmlSetProp (objElem, (xmlChar *) "locked",
+				(xmlChar *) "true");
 
 #define SETSTRING_PROP(field)\
 		  if(((lilydirective *) curObj->object)->field && ((lilydirective *) curObj->object)->field->len)\
@@ -1393,7 +1402,7 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 #undef SETSTRING_PROP
 
 #define SETINT_PROP(x)\
-		    newXMLIntProp (objElem, (xmlChar *) #x,\
+		    if(((lilydirective *) curObj->object)->x) newXMLIntProp (objElem, (xmlChar *) #x, \
 				   (((lilydirective *) curObj->object)->x));
 		    SETINT_PROP (x);
 		    SETINT_PROP (y);
@@ -1403,7 +1412,9 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 		    SETINT_PROP (gy);
 		    SETINT_PROP (minpixels);
 		    SETINT_PROP (override);
-#undef SETINT_PROP		  
+#undef SETINT_PROP
+		    if(curObj->durinticks)
+		      newXMLIntProp (objElem, "ticks", curObj->durinticks);
 		  break;
 		case BARLINE:
 		case MEASUREBREAK:
@@ -1430,10 +1441,13 @@ exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
     }				/* end for each voice in score */
   }// for each movement
   /* Save the file. */
-
+#if 0
   if (xmlSaveFile (filename->str, doc) < 0)
     g_warning ("Could not save file %s", filename->str);
-
+#else
+  if (xmlSaveFormatFile (filename->str, doc, 1) < 0)
+    g_warning ("Could not save file %s", filename->str);
+#endif
   /* Clean up all the memory we've allocated. */
 
   xmlFreeDoc (doc);
