@@ -219,7 +219,7 @@ void fluid_output_midi_event(unsigned char *buffer)
 	gint velocity =  ((gint)(Denemo.gui->si->master_volume * buffer[2]));
 	if(velocity>0x7F) velocity = 0x7F;
 	fluid_synth_noteon(synth, chan,  buffer[1], velocity);
-	//g_print("play %d on %f\n", chan, event->time_seconds);
+	//g_print("play %d on %d\n", chan, velocity);
       }
 	break;
        case NOTE_OFF:
@@ -247,6 +247,18 @@ void fluid_output_midi_event(unsigned char *buffer)
        case MIDI_SYSTEM_RESET:
          fluid_synth_system_reset(synth);
 	 break;
+
+      case SYS_EXCLUSIVE_MESSAGE1:
+	if(FLUIDSYNTH_VERSION_MAJOR>=1 && FLUIDSYNTH_VERSION_MINOR>=1)
+	  fluid_synth_sysex(synth, buffer+1, 19, NULL, 0, NULL, FALSE);
+	else
+	  g_warning("Not supported by this fluidsynth version use >=1.1");
+	//char *response, int *response_len, int *handled, int dryrun)
+	break;
+	
+	//FIXME - duplicate code below 
+      default:
+	g_warning("MIDI message Not handled");
       }
   }
 }
@@ -333,7 +345,7 @@ static gboolean fluidsynth_play_smf_event(gchar *callback)
   } 
   // g_print("rightmost %f event %f\n", si->rightmost_time, event->time_seconds);
   if(si->rightmost_time>0.0 && event->time_seconds>si->rightmost_time)
-     page_viewport();
+    page_viewport(), si->rightmost_time=-1;
   gdouble thetime = get_time() - si->start_player;
   pause_time = thetime;
   //g_print("thetime %f\n", thetime);
@@ -393,6 +405,17 @@ static gboolean fluidsynth_play_smf_event(gchar *callback)
        case MIDI_SYSTEM_RESET:
          fluid_synth_system_reset(synth);
 	 break;
+
+      case SYS_EXCLUSIVE_MESSAGE1:
+	if(FLUIDSYNTH_VERSION_MAJOR>=1 && FLUIDSYNTH_VERSION_MINOR>=1) {
+	 gint ret = fluid_synth_sysex(synth, event->midi_buffer+1, 19, NULL, 0, NULL, FALSE);
+	 //g_print("Got ret %d bytes sent are %x %x %x %x %x\n", ret, event->midi_buffer[1], event->midi_buffer[2], event->midi_buffer[3], event->midi_buffer[4], event->midi_buffer[5]);
+	}
+	else
+	  g_warning("Not supported by this fluidsynth version use >=1.1");
+	//char *response, int *response_len, int *handled, int dryrun)
+	break;
+
       }
   }
   return TRUE;
@@ -480,18 +503,18 @@ fluid_rhythm_feedback(gint duration, gboolean rest, gboolean dot) {
   if(dot)
     fluid_playpitch(67, 100, 9, 60*Denemo.gui->si->master_volume);
   else
-    fluid_playpitch(rhythm_sounds[duration], rest?100:200, 9, 60*Denemo.gui->si->master_volume);
+    fluid_playpitch(rhythm_sounds[duration], rest?100:200, 9, 127*Denemo.gui->si->master_volume);
   //add extra sound effect for rests
   if(rest)
-    fluid_playpitch(46, 300, 9, 30*Denemo.gui->si->master_volume);
-  
-  //  g_print("playing %d %d\n", rhythm_sounds[duration], (60/(4*Denemo.gui->si->tempo*(1<<duration)))*1000);
+    fluid_playpitch(46, 300, 9, 127*Denemo.gui->si->master_volume);
+ 
+  //g_print("playing %d %d\n", rhythm_sounds[duration], (60/(4*Denemo.gui->si->tempo*(1<<duration)))*1000);
 
 }
 
 
 static fluid_midi_driver_t* midi_in;
-
+//Under Interrupt
 static void handle_midi_event(gchar *buf) {
   if(Denemo.gui->midi_destination & MIDIRECORD)
     record_midi(buf,  get_time() - Denemo.gui->si->start_player);
@@ -502,9 +525,20 @@ static void handle_midi_event(gchar *buf) {
 }
 
 
+
+static gchar midi_in_buf[3]; 
+static volatile gboolean midi_in_ready = FALSE;
+static midi_in_timer_id = 0;
+static gboolean midi_in_timer_callback(void) {
+  if(midi_in_ready)
+    handle_midi_event(midi_in_buf);
+  midi_in_ready = FALSE;
+  return TRUE;//timer keeps going
+}
+
+//Under Interrupt
 static void handle_midi_in(void* data, fluid_midi_event_t* event)
 {
-  gchar buf[3];
   int type = 
     fluid_midi_event_get_type(event);
   //g_print("event type: %x\n", type);
@@ -513,33 +547,43 @@ static void handle_midi_in(void* data, fluid_midi_event_t* event)
   case NOTE_OFF:
   case KEY_PRESSURE:
   case CONTROL_CHANGE:
-  case PITCH_BEND:
   case 0xF2:    
     {
       int key = fluid_midi_event_get_key(event);
       int velocity = fluid_midi_event_get_velocity(event);
-      buf[0] = type;
-      buf[1] = key;
-      buf[2] = velocity;
+      midi_in_buf[0] = type;
+      midi_in_buf[1] = key;
+      midi_in_buf[2] = velocity;
       if(type==NOTE_ON && velocity==0) {//Zero velocity NOTEON is used as NOTEOFF by some MIDI controllers
-	buf[0]=NOTE_OFF;
-	buf[2]=128;
+	midi_in_buf[0]=NOTE_OFF;
+	midi_in_buf[2]=127;
       }
+      midi_in_buf[0] |= ((DenemoStaff *)Denemo.gui->si->currentstaff->data)->midi_channel;
       //g_print("key is %d\n", key);
-      handle_midi_event(buf);
+      //handle_midi_event(midi_in_buf);
+      midi_in_ready = TRUE;
+    }
+    break;
+  case PITCH_BEND:
+  
+    {
+      short key = fluid_midi_event_get_pitch(event);
+      midi_in_buf[0] = type;
+      midi_in_buf[1] = key>>7;
+      midi_in_buf[2] = key&0x7F;
+      midi_in_ready = TRUE;
     }
     break;
 
-  
-    
   case PROGRAM_CHANGE:
   case CHANNEL_PRESSURE:
   case 0xF3: 
     {
       int key = fluid_midi_event_get_key(event);
-      buf[0] = type;
-      buf[1] = key;
-      handle_midi_event(buf);
+      midi_in_buf[0] = type;
+      midi_in_buf[1] = key;
+      //handle_midi_event(midi_in_buf);
+      midi_in_ready = TRUE;
     }
     break;
   default:
@@ -562,17 +606,23 @@ fluid_start_midi_in(void)
 #endif
   //g_print("success %d\n", success);
   midi_in = new_fluid_midi_driver(settings, handle_midi_in, NULL);
+
+
   //g_print("midi in on %p\n", midi_in);
-  if(midi_in)
+  if(midi_in) {
+    midi_in_timer_id = g_timeout_add (20, (GSourceFunc) midi_in_timer_callback, NULL);
     return 0;
+  }
   else
     return -1;
 }
 int
 fluid_stop_midi_in(void)
 {
-  if(midi_in)
+  if(midi_in) {
+    g_source_remove(midi_in_timer_id);
     delete_fluid_midi_driver(midi_in);
+  }
   else
     return -1;
   midi_in = NULL;
