@@ -27,6 +27,9 @@
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
+static gint version_number;
+
+
 static gint current_movement = 0, current_staff=0, current_measure=0, current_position = 0;
 
 /* Defines for making traversing XML trees easier */
@@ -262,7 +265,7 @@ parseDirective (xmlNodePtr parentElem, xmlNsPtr ns,
     if(ELEM_NAME_EQ (childElem, "graphic_name")) {
       directive->graphic_name =  g_string_new((gchar *)xmlNodeListGetString (childElem->doc,\
 						  childElem->xmlChildrenNode, 1));
-      loadGraphicItem(directive->graphic_name->str, (GdkBitmap**)&directive->graphic,  &directive->width, &directive->height);
+      loadGraphicItem(directive->graphic_name->str, (DenemoGraphic**)&directive->graphic);
       /* FIXME,handle not loaded */
     }      
   }
@@ -325,6 +328,17 @@ parseVerses (DenemoScore *si, DenemoStaff *staff, xmlNodePtr parentElem, xmlNsPt
   return verses;
 }
 
+/* Fix for semantic change to prefix field in the directives */
+static fix_prefix_use(GList *directives) {
+  GList *g;
+  for(g=directives;g;g=g->next) {
+    DenemoDirective *directive = g->data;
+    if(directive->prefix) {
+      directive->prefix = g_string_new(g_strdup_printf("%%{Disabled form \n%s\n use newer command %%}\n", directive->prefix->str));
+      directive->display = g_string_new(g_strdup_printf("Warning - re-run the %s command here!\nold version", directive->tag->str));
+    }
+  }
+}
 static GList *
 parseDirectives (xmlNodePtr parentElem, xmlNsPtr ns)
 {
@@ -965,7 +979,8 @@ DenemoObject *chordObj =
   FOREACH_CHILD_ELEM (childElem, chordElem)  {
     if (childElem->ns == ns) {
       if (ELEM_NAME_EQ (childElem, "directives"))  {
-	((chord *) chordObj->object)->directives = parseDirectives(childElem, ns);	  
+	((chord *) chordObj->object)->directives = parseDirectives(childElem, ns);
+	if(version_number<3) fix_prefix_use(((chord *) chordObj->object)->directives);
       }
       else if (ELEM_NAME_EQ (childElem, "figure"))
 	{
@@ -1216,6 +1231,11 @@ parseChord (xmlNodePtr chordElem, xmlNsPtr ns,
 	  {
 	    /* This was already parsed during parseBaseChord(). */
 	  }
+	else if (ELEM_NAME_EQ (childElem, "ticks"))
+	  {
+	    chordObj->basic_durinticks = getXMLIntChild (childElem);
+	    ((chord *)chordObj->object)->baseduration = -chordObj->basic_durinticks;
+	  }
 	else if (ELEM_NAME_EQ (childElem, "dynamic"))
 	  {
 	    parseDynamic (childElem, chordObj);
@@ -1508,7 +1528,7 @@ parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
   GET_STR_FIELD(graphic_name);
   GET_STR_FIELD(prefix);
   if(thedirective->graphic_name && thedirective->graphic_name->len)
-    loadGraphicItem(thedirective->graphic_name->str ,(GdkBitmap**) &thedirective->graphic, &thedirective->width, &thedirective->height);
+    loadGraphicItem(thedirective->graphic_name->str ,(DenemoGraphic**) &thedirective->graphic);
 #define GET_INT_FIELD(x)\
   gchar *x = (gchar *) xmlGetProp (LilyDirectiveElem, (xmlChar *) #x);\
   if(x)\
@@ -1525,6 +1545,10 @@ parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
   GET_INT_FIELD(minpixels);
   // curobj->minpixelsalloted = thedirective->minpixels?thedirective->minpixels:16;//FIXME setpixelmin
   setpixelmin(curobj);
+  gchar *ticks = (gchar *)xmlGetProp (LilyDirectiveElem, (xmlChar *)"ticks");
+  if(ticks)
+    curobj->durinticks = atoi(ticks);
+  //curobj->baseduration = - curobj->durinticks;
   return curobj;
 }
 
@@ -2821,17 +2845,16 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
       ret = -1;
       goto cleanup;
     }
-  gint version_number = 2;
-  if (strcmp (version, "2.0") != 0)
-    {
-      if(strcmp (version, "1.0") != 0) {
-      g_warning ("Denemo XML file version %s found, but we can only handle "
-		 "versions 1.0 and 2.0", version);
+
+
+  sscanf(version, "%d", &version_number);
+  if(version_number > CURRENT_XML_VERSION)
+     {
+      g_warning ("Denemo XML file version %d found, but we can only handle %d and lower"
+		 , version_number, CURRENT_XML_VERSION);
       ret = -1;
       goto cleanup;
-      } else
-	version_number = 1;
-    }
+      }
 
   /*
    * Okay, we've got a bona fide, 100% genuine Denemo XML file (hopefully).
@@ -2841,19 +2864,15 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
 
   buildXMLIDToElemMap (doc);
   /* Then, parse the score. */
-  if(version_number==2) {
+  if(version_number>=2) {
     xmlNodePtr childElem;
     switch(type) {
     case ADD_STAFFS:
       FOREACH_CHILD_ELEM(childElem, rootElem){
-	if (ELEM_NAME_EQ (childElem, "lilycontrol") 
-	    || ELEM_NAME_EQ (childElem, "custom_scoreblock")
-	    || ELEM_NAME_EQ (childElem, "visible_scoreblock")
-	    || ELEM_NAME_EQ (childElem, "scoreheader-directives")
-	    || ELEM_NAME_EQ (childElem, "paper-directives")){
+	if (ELEM_NAME_EQ (childElem, "movement")) 
+	  ret |=  parseMovement(childElem, ns, gui, type);
+	else
 	  continue;
-	} else
-	ret |=  parseMovement(childElem, ns, gui, type);
 	//g_print("parsed more staffs breaking now\n");
 	break;//Note: we only adds staffs from first movement
       }
@@ -2940,6 +2959,8 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
 			  point_to_empty_movement(gui);
 			  ret |=  parseMovement(childElem, ns, gui, type);
 			}
+  if(gui->si && gui->si->lyricsbox)
+    gtk_widget_hide(gui->si->lyricsbox);
       }
 	
 	if(getNumCharsSchemeText())
@@ -2972,9 +2993,18 @@ importXML (gchar * filename, DenemoGUI *gui, ImportType type)
       goto cleanup;
     }
   }
-
-  while(--current_movement>0)
+  if(gui->si->lyricsbox)
+    gtk_widget_hide(gui->si->lyricsbox);
+  while(--current_movement>0) {
+    if(gui->si->lyricsbox)
+       gtk_widget_hide(gui->si->lyricsbox);
     next_movement(NULL, NULL);
+  }
+  if(gui->si->lyricsbox)
+    if(!Denemo.prefs.lyrics_pane)
+      gtk_widget_hide(gui->si->lyricsbox);
+    else
+      gtk_widget_show(gui->si->lyricsbox);
   score_status(gui, FALSE);
 
  cleanup:
