@@ -263,6 +263,7 @@ insert_chord_key (DenemoGUI * gui)
 void
 go_to_A_key (DenemoGUI * gui)
 {
+ Denemo.gui->last_source = INPUTKEYBOARD;
   shiftcursor (gui, 5);
 }
 
@@ -273,6 +274,7 @@ go_to_A_key (DenemoGUI * gui)
 void
 go_to_B_key (DenemoGUI * gui)
 {
+ Denemo.gui->last_source = INPUTKEYBOARD;
   shiftcursor (gui, 6);
 }
 
@@ -283,6 +285,7 @@ go_to_B_key (DenemoGUI * gui)
 void
 go_to_C_key (DenemoGUI * gui)
 {
+ Denemo.gui->last_source = INPUTKEYBOARD;
   shiftcursor (gui, 0);
 }
 
@@ -303,6 +306,7 @@ go_to_D_key (DenemoGUI * gui)
 void
 go_to_E_key (DenemoGUI * gui)
 {
+ Denemo.gui->last_source = INPUTKEYBOARD;
   shiftcursor (gui, 2);
 }
 
@@ -313,6 +317,7 @@ go_to_E_key (DenemoGUI * gui)
 void
 go_to_F_key (DenemoGUI * gui)
 {
+ Denemo.gui->last_source = INPUTKEYBOARD;
   shiftcursor (gui, 3);
 }
 
@@ -323,6 +328,7 @@ go_to_F_key (DenemoGUI * gui)
 void
 go_to_G_key (DenemoGUI * gui)
 {
+ Denemo.gui->last_source = INPUTKEYBOARD;
   shiftcursor (gui, 4);
 }
 
@@ -342,10 +348,14 @@ octave_shift_key (DenemoGUI * gui, gint amount)
 	  objnode *thenote = nearestnote (gui->si->currentobject->data, gui->si->cursor_y);
 	  if(thenote) {
 	    note copy = *((note *) thenote->data);
-	    delete_chordnote(gui);
+	    GList *direcs = ((note *)thenote->data)->directives;
+	    delete_chordnote(gui);//does not delete the directives.
 	    gui->si->cursor_y = copy.mid_c_offset + amount;
 	    insert_chordnote(gui);
 	    changeenshift(gui->si->currentobject->data, gui->si->cursor_y, copy.enshift);
+	    thenote = nearestnote (gui->si->currentobject->data, gui->si->cursor_y);
+	    if(thenote)
+	      ((note *)thenote->data)->directives = direcs;
 	    score_status(gui, TRUE);
 	  }
 	}
@@ -714,6 +724,9 @@ deletepreviousobject(DenemoGUI * gui)
 	{
 #define CURRP ((RhythmPattern *)gui->currhythm->data) 
 	  g = g->prev; /* list is circular - should we stop at beginning? */
+	  if(gui->cstep) {
+	    gui->cstep = gui->cstep->prev?gui->cstep->prev:g_list_last(CURRP->clipboard)->data;	    
+	  }
 	  if(((RhythmElement*)g->data)->icon) {
 	    GtkWidget *label = LABEL(CURRP->button);
 	    gtk_label_set_markup(GTK_LABEL(label),((RhythmElement*)g->data)->icon);
