@@ -51,6 +51,7 @@
 struct DenemoRoot Denemo;
 
 /* set strings for midi_audio_output */
+const gchar *None = "None";
 const gchar *Jack = "Jack";
 const gchar *Portaudio = "Portaudio";
 const gchar *Fluidsynth = "InternalSynth";
@@ -505,18 +506,20 @@ main (int argc, char *argv[])
 
   register_stock_items ();
 
-#define choice1  "Default\nUse this until you have read the manual\n"
-#define choice4 "Classic\nOld Denemo pc-keyboard interface."
+#define choice1 "Simple\nQuick start users: use this until you have read the manual\n"
 #define choice2 "Arranger\nExperienced Users: transcribing music, playing music in, transposing etc"
 #define choice3 "Composer\nExperienced Users: entering and modifying music, working with selections WASD use etc"
+#define choice4 "Classic\nOld Denemo pc-keyboard interface."
+#define choice5 "LilyPond\nExperienced Users with LilyPond knowledge"
+#define choice6 "AllCommands\nUsers wanting to see the complete command set. No pre-defined shortcuts"
 
   if(uses_default_commandset()) {
     // infodialog("Nearly every menu item can be right-clicked, for help, setting keyboard shortcuts and more"); this  should always appear on top of the main window, but it is unresponsive to dismissal at first.
     // get_option returns a pointer into the string passed in
-    gchar *choice = get_option(choice1"\0"choice2"\0"choice3"\0"choice4"\0", strlen(choice1)+1+strlen(choice2)+1+strlen(choice3)+1+strlen(choice4)+1);
+    gchar *choice = get_option(choice1"\0"choice2"\0"choice3"\0"choice4"\0"choice5"\0"choice6"\0", strlen(choice1)+1+strlen(choice2)+1+strlen(choice3)+1+strlen(choice4)+1+strlen(choice5)+1+strlen(choice6)+1);
     if(choice==NULL)
       choice = choice1;
-    if(choice!=choice1) {
+    if(strcmp(choice, choice1)) {
       choice = g_strdup(choice);
       gchar *c;
       for(c=choice;*c;c++)
@@ -643,6 +646,7 @@ if (Denemo.prefs.midi_audio_output == Jack)
 if (Denemo.prefs.midi_audio_output == Fluidsynth)
   fluidsynth_init(); 
 #endif
+#ifdef _HAVE_PORTAUDIO_
 if (Denemo.prefs.midi_audio_output == Portaudio){
   /* Immediate Playback */
   if(Denemo.prefs.immediateplayback) {
@@ -651,10 +655,11 @@ if (Denemo.prefs.midi_audio_output == Portaudio){
 				immediate playback */
       //g_print("Initializing audio out\n");
       init_audio_out();
-
     }
   }
 }
+#endif    
+
 
 
   /* Set up the signal handler */
