@@ -13,7 +13,7 @@
 #include <math.h>
 #include <string.h>
 #include "smf.h"
-
+#include "pitchentry.h"
 
 #define MIDI_NOTEOFF		0x80
 #define MIDI_NOTEON		0x90
@@ -100,6 +100,42 @@ midi_init ()
   return 0;
 }
 
+/* change the MIDI output tuning */
+void change_tuning(gdouble *cents) {
+  gchar buffer[] = {
+    0xF0, 0x7F,// 		Universal Real-Time SysEx header
+
+    0x7F, //<device ID> 	ID of target device (7F = all devices)
+
+    0x08,// 		sub-ID#1 = "MIDI Tuning Standard"
+	
+    0x08,// 		sub-ID#2 = "scale/octave tuning 1-byte form (Real-Time)"
+
+    0x03, /*		channel/options byte 1
+			bits 0 to 1 = channel 15 to 16
+			bit 2 to 6 = reserved for future expansion*/
+
+    0x7F, // 		channel byte 2 - bits 0 to 6 = channel 8 to 14
+
+    0x7F, //		channel byte 3 - bits 0 to 6 = channel 1 to 7
+    0,0,0,0,0,0,0,0,0,0,0,0,
+    /*	[ss]		12 byte tuning offset of 12 semitones from C to B
+				00H means -64 cents
+				40H means 0 cents (equal temperament)
+				7FH means +63 cents */
+
+    0xF7	//	EOX
+
+  };
+  gint i;
+  for(i=0;i<12;i++)
+    buffer[i+8] = 64 + (cents[i]+0.5);
+  if (Denemo.prefs.midi_audio_output == Jack)
+    jack_output_midi_event(buffer, 0, 0);
+  else if (Denemo.prefs.midi_audio_output == Fluidsynth)
+    fluid_output_midi_event(buffer);
+}
+
 void playpitch(double pitch, double duration, double volume, int channel) {
   if(!Denemo.prefs.immediateplayback)
     return;
@@ -122,11 +158,11 @@ void play_midikey(gint key, double duration, double volume, gint channel){
  *  (a g_list_foreach function)
  */
 static void
-playnote (gpointer tone, gpointer chord, int channel)
+playnote (note * tone, /*GList * chord, */int channel)
 {
   gint offset;
   gchar key;
-  gint voice;
+  //gint voice;
   /* Because mid_c_offset is a measure of notes and we need a measure of
    * half-steps, this array will help */
   const gint key_offset[] = { -10, -8, -7, -5, -3, -1, 0, 2, 4, 5, 7, 9, 11 };
@@ -136,25 +172,24 @@ playnote (gpointer tone, gpointer chord, int channel)
   /* 60 is middle-C in MIDI keys */
   key = 60 + 12 * (offset / 7) + key_offset[offset % 7 + 6];
   key += ((note *) tone)->enshift;
-  voice = g_list_index ((GList *) chord, tone);
+  //voice = g_list_index ((GList *) chord, tone);
 
   play_midikey(key, 0.3, 0.5/*Denemo.prefs.pcmvolume*/, channel);
 
 }
 
 /** 
- * This version of the function opens and closes /dev/sequencer w/ each
- * write, as a separate process for performance-type reasons 
+
  */
 void
-playnotes (gboolean doit, chord chord_to_play, int channel)
+playnotes (gboolean doit, chord *chord_to_play, int channel)
 {
   //g_print("playnotes called for channel %d\n", channel);
-  if (doit && (sequencer_absent) && chord_to_play.notes) {
-    playnote( chord_to_play.notes->data, chord_to_play.notes, channel);
-    return;
+  if (doit && (sequencer_absent) && chord_to_play->notes) {
+    GList *g;
+    for(g=chord_to_play->notes;g;g=g->next)
+      playnote( g->data, /*chord_to_play->notes,*/ channel);
   }
-
 }
 
 // MIDI input
@@ -309,9 +344,9 @@ gboolean set_midi_capture(gboolean set) {
   return ret;
 }
 
-#define command ((*buf)&0xFF)
-#define notenumber ((*(buf+1))&0xFF)
-#define velocity ((*(buf+2))&0xFF)
+#define command ((*buf)&0xF0)
+#define notenumber ((*(buf+1))&0x7F)
+#define velocity ((*(buf+2))&0x7F)
 void process_midi_event(gchar *buf) {
   //g_print("process midi (%s) %x %x %x\n",divert_midi_event?"diverted":"straight", command, notenumber, velocity);
   if(divert_midi_event &&  divert_midi_id==Denemo.gui->id){
@@ -324,7 +359,7 @@ void process_midi_event(gchar *buf) {
 
   if(command==MIDI_NOTEON && velocity==0) {//Zero velocity NOTEON is used as NOTEOFF by some MIDI controllers
     buf[0]=MIDI_NOTEOFF;
-    buf[2]=128;
+    buf[2]=128;//FIXME 127
   }
   
   if(midi_capture_on) {
@@ -335,6 +370,20 @@ void process_midi_event(gchar *buf) {
   } else {
     if(command==MIDI_NOTEON)
       store_midi_note(notenumber);
+    else if(command==MIDI_CTL_CHANGE) {
+      gchar *command_name = get_midi_control_command(notenumber, velocity);
+      if(command_name)
+      //FIXME what sort of free? g_free(command_name);
+	execute_callback_from_name(Denemo.map, command_name);
+
+    } else if(command==MIDI_PITCH_BEND) {
+      gchar *command_name = get_midi_pitch_bend_command((notenumber<<7) + velocity);
+      if(command_name)
+      //FIXME what sort of free? g_free(command_name);
+	execute_callback_from_name(Denemo.map, command_name);
+
+      ;//the same
+    }
   }
 }
 
