@@ -13,7 +13,7 @@ gint midi_init ();
 gint get_midi_channel();
 gint get_midi_prognum();
 gint get_midi_port();
-void playnotes (gboolean doit, chord chord_to_play,int prognum);
+void playnotes (gboolean doit, chord *chord_to_play,int prognum);
 
 void play_midikey(gint key, double duration, double volume, gint channel);
 void process_midi_event(gchar *buf);
