@@ -1,6 +1,6 @@
 /* mwidthdialog.cpp
  * a callback that creates a dialog box prompting the user to change the
- * staff height
+ * measure width - this is a denemo display-only thing(?)
 
  * for Denemo, a gtk+ frontend to GNU Lilypond
  * (c) 1999-2005 Matthew Hiller */
@@ -43,8 +43,9 @@ set_mwidth (GtkWidget * widget, gpointer data)
  *
  */
 void
-score_mwidth_change (GtkAction * action, DenemoGUI * gui)
+score_mwidth_change (GtkAction *action, gpointer param)
 {
+  DenemoGUI *gui = Denemo.gui;
   GtkWidget *dialog;
   GtkWidget *label;
   GtkWidget *textentry;
