@@ -223,6 +223,7 @@ DenemoDirective *clone_directive(DenemoDirective *directive) {
   CLONE(postfix);
   CLONE(display);
   CLONE(graphic_name);
+  CLONE(midibytes);
 #undef CLONE
   if(directive->graphic) {
     ret->graphic = directive->graphic;//alternatively could load it via loadGraphicItem, is the same
@@ -334,6 +335,9 @@ dnm_clone_object (DenemoObject * orig)
 	    case LILYDIRECTIVE: {
 	      lilydirective *curlily = (lilydirective*)orig->object;
 	      ret = directive_object_new(clone_directive(curlily));
+	      ret->durinticks = orig->durinticks;
+	      ret->basic_durinticks = orig->basic_durinticks;
+	      
 	    }
 	      break;
 	    default:
