@@ -44,9 +44,13 @@ static gdouble lowest_pitch = DEFAULT_LOW;
 static gdouble highest_pitch = DEFAULT_HIGH;
 static gboolean repeated_notes_allowed;
 static gdouble transposition_required = 1.0;
+
+static enharmonic_position = 0;
+
 typedef struct notespec {
   guint step;  /* 0=C 1=D ... 6=B */
   gint alteration; /* -2 = bb, -1=b, 0=natural, 1=#, 2=x */
+  gint cents;//deviation from Equal
 } notespec;
 
 typedef struct notepitch {
@@ -58,11 +62,14 @@ typedef struct notepitch {
 
 typedef struct temperament {
   gchar *name;
-  gint sharp;/* which notepitch is the sharpest in circle of 5ths - initially G# */
-  gint flat;/* which notepitch is the flattest in circle of 5ths - initially Eb */
+  gint unusedsharp;/* which notepitch is the sharpest in circle of 5ths - initially G# */
+  gint unusedflat;/* which notepitch is the flattest in circle of 5ths - initially Eb */
   notepitch notepitches[12]; /* pitches of C-natural, C#, ... A#, B-natural */
 } temperament;
 
+static gint flat_degree=3, sharp_degree=8;/* which notepitch is the sharpest in circle of 5ths - initially G#
+                                     flattest in circle of 5ths - initially Eb */
+static gint temperament_offset=0;/* shift in temperament around circle of 5ths */
 #if 0
 Pythagorean
 261.6	279.4	294.3 	310.1	331.1	348.8 	372.5	392.4	419.1 	441.5	465.1	496.7
@@ -207,8 +214,8 @@ static temperament *temperaments[] = {&Equal, &Meantone, &WerckmeisterIV, &Lehma
 
 
 static void switch_back_to_main_window(void) {
-
-    gtk_window_present(GTK_WINDOW(Denemo.window));
+  gtk_window_present(GTK_WINDOW(Denemo.window));
+  gtk_widget_grab_focus (Denemo.gui->scorearea);
 }
 
 
@@ -267,10 +274,10 @@ static gchar * notenames(gpointer p) {
   gint i;
   oldstr = g_strdup("");
   for(i=0;i<12;i++) {
-    if(i==t->flat)
+    if(i==flat_degree)
       str = g_strdup_printf("%s<span size=\"large\" foreground=\"red\">%c%s</span> ", oldstr, step_name(t->notepitches[i].spec.step), alteration_name(t->notepitches[i].spec.alteration));
     else
-    if(i==t->sharp)
+    if(i==sharp_degree)
        str = g_strdup_printf("%s<span size=\"large\" foreground=\"blue\">%c%s</span> ", oldstr, step_name(t->notepitches[i].spec.step), alteration_name(t->notepitches[i].spec.alteration));
     else
       str = g_strdup_printf("%s%c%s ", oldstr, step_name(t->notepitches[i].spec.step), alteration_name(t->notepitches[i].spec.alteration));
@@ -316,10 +323,10 @@ static gchar *nameof(gint notenumber) {
 
 //Caller must free
 gchar *sharpest(void) {
-  return nameof(PR_temperament->sharp);
+  return nameof(sharp_degree);
 }
 gchar *flattest(void) {
-  return nameof(PR_temperament->flat);
+  return nameof(flat_degree);
 }
 
 /* returns an opaque id for the user's default temperament
@@ -333,15 +340,11 @@ static gpointer default_temperament() {
   return (gpointer) &Equal;
 }
 
-static void set_temperament(GtkMenuItem *item, temperament *t) {
-  PR_temperament = t;
-  g_string_assign(Denemo.prefs.temperament, t->name);
-  //g_print("Temperament set to %p (cf %p %p)\n", PR_temperament, &Equal, &Meantone);
-}
 
 static void sharpen(GtkButton *button, GtkWidget *label) {
-#define f  (PR_temperament->notepitches[PR_temperament->flat].spec)
-  gint next = (PR_temperament->flat+11)%12;
+  enharmonic_position++;
+#define f  (PR_temperament->notepitches[flat_degree].spec)
+  gint next = (flat_degree+11)%12;
 #define g (PR_temperament->notepitches[next].spec)
  if(g.alteration+1>2) 
   return;
@@ -353,14 +356,16 @@ static void sharpen(GtkButton *button, GtkWidget *label) {
   }
 #undef f
 #undef g
-  PR_temperament->sharp = PR_temperament->flat;
-  PR_temperament->flat = (PR_temperament->flat+7)%12;
+  sharp_degree = flat_degree;
+  flat_degree = (flat_degree+7)%12;
+  temperament_offset = (temperament_offset+5)%12;
   if(1 /*PR_window*/) {
     gchar *names = notenames(PR_temperament);
     gtk_label_set_markup((GtkLabel*)label, names);
     g_free(names);
     switch_back_to_main_window();
   }
+  reset_temperament();
   return;
 }
 
@@ -369,9 +374,9 @@ static void sharpen(GtkButton *button, GtkWidget *label) {
 
 
 static void flatten(GtkButton *button, GtkWidget *label) {
-
-#define s (PR_temperament->notepitches[PR_temperament->sharp].spec)
-  gint next = (PR_temperament->sharp+1)%12;
+  enharmonic_position--;
+#define s (PR_temperament->notepitches[sharp_degree].spec)
+  gint next = (sharp_degree+1)%12;
 #define t (PR_temperament->notepitches[next].spec)
 if(t.alteration-1<-2) 
   return;
@@ -381,17 +386,28 @@ if(t.alteration-1<-2)
  }
 #undef s
 #undef t
-  PR_temperament->flat = PR_temperament->sharp;
-  PR_temperament->sharp = (PR_temperament->sharp+5)%12;
+  flat_degree = sharp_degree;
+  sharp_degree = (sharp_degree+5)%12;
+  temperament_offset = (temperament_offset+7)%12;
   if(1/*PR_window*/) {
   gchar *names = notenames(PR_temperament);
   gtk_label_set_markup((GtkLabel*)label, names);
   g_free(names);
   switch_back_to_main_window();
   }
+  reset_temperament();
   return;
 }
 
+void
+set_enharmonic_position(gint position) {
+  while (position<enharmonic_position) {
+    set_flatter(NULL, NULL);
+  }
+  while (position>enharmonic_position) {
+    set_sharper(NULL, NULL);
+  }
+}
 static void enharmonic_step (gboolean sharp) {
   gchar *sharpestname, *flattestname;
   if(sharp)
@@ -401,8 +417,8 @@ static void enharmonic_step (gboolean sharp) {
   GtkAction *sharpaction = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/SharpenEnharmonicSet");
   GtkAction *flataction = gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/FlattenEnharmonicSet");
 
-  sharpestname = nameof(PR_temperament->sharp);
-  flattestname = nameof(PR_temperament->flat);
+  sharpestname = nameof(sharp_degree);
+  flattestname = nameof(flat_degree);
   gchar *label = g_strdup_printf("(%s) Step Sharper", sharpestname);
   GValue a = {0};
   g_value_init (&a, G_TYPE_STRING);
@@ -427,7 +443,7 @@ void set_flatter(GtkAction *action, gpointer param) {
 void
 signal_measure_end(void) {
 if (Denemo.prefs.midi_audio_output == Fluidsynth)
-  fluid_playpitch(74, 300, 9, (gint)(40*Denemo.gui->si->master_volume));
+  fluid_playpitch(74, 300, 9, (gint)(100*Denemo.gui->si->master_volume));
  else
    gdk_beep();
 }
@@ -1011,6 +1027,38 @@ static void toggle_tuning(GtkToggleButton *button, DenemoGUI *gui) {
       id = 0;
     }
 }
+#endif
+
+/* return an array of values representing deviations from equal temperament for 12 notes from C for the passed temperament. Returned value is read only */
+static gdouble *get_cents(temperament *t) {
+  static gdouble array[12];
+  int i, j;
+  for(i=0;i<12;i++) {
+    j = (i+temperament_offset)%12;
+    //g_print("cents tempered %d to %d unshifted %f shifted %f\n", i, j, 1200 * log2(t->notepitches[i].pitch/Equal.notepitches[i].pitch), 1200 * log2(t->notepitches[j].pitch/Equal.notepitches[j].pitch));
+    array[i] = 1200 * log2(t->notepitches[j].pitch/Equal.notepitches[j].pitch);
+  }
+  return array;
+}
+
+/* return a string off offsets from 64 representing cents deviation for current temperment
+ caller must free*/
+gchar *get_cents_string(void) {
+  gdouble *values = get_cents(PR_temperament);
+  return  g_strdup_printf(" %d %d %d %d %d %d %d %d %d %d %d %d ", (gint)(64+values[0])&0x7f,
+		  (gint)(64+values[1])&0x7f,(gint)(64+values[2])&0x7f,(gint)(64+values[3])&0x7f,(gint)(64+values[4])&0x7f,(gint)(64+values[5])&0x7f,(gint)(64+values[6])&0x7f,(gint)(64+values[7])&0x7f,(gint)(64+values[8])&0x7f,(gint)(64+values[9])&0x7f,(gint)(64+values[10])&0x7f,(gint)(64+values[11])&0x7f);
+}
+
+gchar *get_sharpest(void) {
+  return nameof(sharp_degree);
+}
+gchar *get_flattest(void) {
+  return nameof(flat_degree);
+}
+gchar *get_temperament_name(void) {
+  return g_strdup(PR_temperament->name);
+}
+
 #define COLUMN_NAME (0)
 #define COLUMN_PTR (1)
 
@@ -1019,10 +1067,16 @@ static void  temperament_changed_callback (GtkComboBox *combobox,  GtkListStore
   gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combobox), &iter);
   gtk_tree_model_get (GTK_TREE_MODEL (list_store), &iter,
 		      COLUMN_PTR, &PR_temperament, -1);
+  change_tuning(get_cents(PR_temperament));
   g_string_assign(Denemo.prefs.temperament, PR_temperament->name);
+  switch_back_to_main_window();
+}
+
+void
+reset_temperament(void) {
+change_tuning(get_cents(PR_temperament));
 }
 
-#endif
 GtkWidget *get_enharmonic_frame(void) {
   static GtkWidget *frame;
   if(frame==NULL) {
@@ -1055,6 +1109,42 @@ GtkWidget *get_enharmonic_frame(void) {
   return frame;
 }
 
+GtkWidget *get_temperament_combo(void) {
+  static GtkWidget *combobox;
+  if(combobox==NULL) {
+   
+    
+  GtkListStore *list_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
+  GtkCellRenderer *renderer;
+  combobox = gtk_combo_box_new_with_model (GTK_TREE_MODEL (list_store));
+  g_object_ref(combobox);
+   
+  int i;
+  for (i = 0; i < (gint) G_N_ELEMENTS (temperaments); i++)
+    {GtkTreeIter iter;
+      gtk_list_store_append (list_store, &iter);
+      gtk_list_store_set (list_store, &iter,
+			  COLUMN_NAME, temperaments[i]->name,
+			  COLUMN_PTR, temperaments[i], -1);
+
+      if((i==0) || (Denemo.prefs.temperament && !strcmp(Denemo.prefs.temperament->str, temperaments[i]->name)))
+	 gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combobox), &iter); 
+    }
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox), renderer, TRUE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combobox),
+				 renderer, "text", COLUMN_NAME);
+  g_signal_connect(G_OBJECT (combobox),  "changed",
+		    G_CALLBACK (temperament_changed_callback), list_store);
+
+
+  }
+  GtkContainer *cont = gtk_widget_get_parent(combobox);
+  if(cont)
+    gtk_container_remove(cont, combobox);
+  return combobox;
+}
+
 #ifdef _HAVE_PORTAUDIO_
 
 static void create_pitch_recognition_window(DenemoGUI *gui) {
@@ -1341,31 +1431,10 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
 		    G_CALLBACK (frequency_smoothing), NULL);
  
 
-  GtkListStore *list_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);
-  GtkCellRenderer *renderer;
-  GtkWidget *combobox = gtk_combo_box_new_with_model (GTK_TREE_MODEL (list_store));
-  int i;
-  for (i = 0; i < (gint) G_N_ELEMENTS (temperaments); i++)
-    {GtkTreeIter iter;
-      gtk_list_store_append (list_store, &iter);
-      gtk_list_store_set (list_store, &iter,
-			  COLUMN_NAME, temperaments[i]->name,
-			  COLUMN_PTR, temperaments[i], -1);
-
-      if(!strcmp(Denemo.prefs.temperament->str, temperaments[i]->name))
-	 gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combobox), &iter); 
-    }
-  renderer = gtk_cell_renderer_text_new ();
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox), renderer, TRUE);
-  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combobox),
-				 renderer, "text", COLUMN_NAME);
-  g_signal_connect(G_OBJECT (combobox),  "changed",
-		    G_CALLBACK (temperament_changed_callback), list_store);
-  gtk_box_pack_start (GTK_BOX (hbox ),
-		      combobox, TRUE, TRUE, 0);
-
+  frame = get_temperament_combo();
+  if(!gtk_widget_get_parent(frame))
+    gtk_container_add (GTK_CONTAINER (hbox), frame);
   }
-
 /* now show the window, but leave the main window with the focus */
   gtk_window_set_focus_on_map((GtkWindow *)PR_window, FALSE);
   gtk_widget_show_all(PR_window);
@@ -1376,7 +1445,7 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   // sort out the size of the drawing area
   // sort out how to pass the deviation from in-tune to the draw_indicator function
   
-}
+  }
 
 
 gint setup_pitch_input(void){
