@@ -34,4 +34,13 @@ gchar *flattest(void);
 void signal_measure_end(void);
 gboolean pitch_entry_active(DenemoGUI *gui);
 GtkWidget *get_enharmonic_frame(void);
+GtkWidget *get_temperament_combo(void);
+void change_tuning(gdouble *cents);
+void reset_temperament(void);
+gchar *get_fents_string(void);
+gchar *get_sharpest(void);
+gchar *get_flattest(void);
+gchar *get_temperament_name(void);
+void set_enharmonic_position(gint position);
+
 #endif //PITCHENTRY_H
