@@ -5,12 +5,15 @@
 gchar *
 get_midi_audio_pointer(gchar *audio_device);
 
+
 void
-playback_local (gpointer callback_data, guint callback_action,
-		GtkWidget * widget);
-void
-ext_midi_playback (GtkAction *action, gpointer param);
+ext_midi_playback (GtkAction *action, DenemoScriptParam *param);
 void
 stop_midi_playback (GtkAction *action, gpointer param);
-
+void 
+playback_panic(void);
+void
+PlaybackRangeDialog();
+void
+MasterVolumeDialog();
 #endif //PLAYBACKH
